@@ -8,16 +8,18 @@ testsuite.
 import contextlib
 import gc as _gc
 import os as _os
-import plistlib as _pl
 import re as _re
 import struct as _struct
 import sys as _sys
+import typing as _typing
 import unittest as _unittest
 import subprocess as _subprocess
+import pickle as _pickle
 from distutils.sysconfig import get_config_var as _get_config_var
 
 import objc
 
+
 # Ensure that methods in this module get filtered in the tracebacks
 # from unittest
 __unittest = False
@@ -175,109 +177,11 @@ def os_release():
     if _os_release is not None:
         return _os_release
 
-    if hasattr(_pl, "load"):
-        with open("/System/Library/CoreServices/SystemVersion.plist", "rb") as fp:
-            pl = _pl.load(fp)
-    else:
-        pl = _pl.readPlist("/System/Library/CoreServices/SystemVersion.plist")
-    v = pl["ProductVersion"]
-    if v.startswith("10.16"):
-        # We're in a python compiled with an pre-macOS 11 SDK, which causes
-        # the system to ly to us. Use sw_vers to get the actual version.
-        v = _subprocess.check_output(["sw_vers", "-productVersion"]).decode().strip()
+    _os_release = (
+        _subprocess.check_output(["sw_vers", "-productVersion"]).decode().strip()
+    )
 
-    return ".".join(v.split("."))
-
-
-def is32Bit():
-    """
-    Return True if we're running in 32-bit mode
-    """
-    if _sys.maxsize > 2 ** 32:
-        return False
-    return True
-
-
-def onlyIf(expr, message=None):
-    """
-    Usage::
-
-        class Tests (unittest.TestCase):
-
-            @onlyIf(1 == 2)
-            def testUnlikely(self):
-                pass
-
-    The test only runs when the argument expression is true
-    """
-
-    def callback(function):
-        if not expr:
-            if hasattr(_unittest, "skip"):
-                return _unittest.skip(message)(function)
-            return lambda self: None  # pragma: no cover (py2.6)
-        else:
-            return function
-
-    return callback
-
-
-def onlyPython2(function):
-    """
-    Usage:
-        class Tests (unittest.TestCase):
-
-            @onlyPython2
-            def testPython2(self):
-                pass
-
-    The test is only executed for Python 2.x
-    """
-    return onlyIf(_sys.version_info[0] == 2, "python2.x only")(function)
-
-
-def onlyPython3(function):
-    """
-    Usage:
-        class Tests (unittest.TestCase):
-
-            @onlyPython3
-            def testPython3(self):
-                pass
-
-    The test is only executed for Python 3.x
-    """
-    return onlyIf(_sys.version_info[0] == 3, "python3.x only")(function)
-
-
-def onlyOn32Bit(function):
-    """
-    Usage::
-
-        class Tests (unittest.TestCase):
-
-            @onlyOn32Bit
-            def test32BitOnly(self):
-                pass
-
-    The test runs only on 32-bit systems
-    """
-    return onlyIf(is32Bit(), "32-bit only")(function)
-
-
-def onlyOn64Bit(function):
-    """
-    Usage::
-
-        class Tests (unittest.TestCase):
-
-            @onlyOn64Bit
-            def test64BitOnly(self):
-                pass
-
-    The test runs only on 64-bit systems
-    """
-    return onlyIf(not is32Bit(), "64-bit only")(function)
+    return _os_release
 
 
 def arch_only(arch):
@@ -293,7 +197,7 @@ def arch_only(arch):
     """
 
     def decorator(function):
-        return onlyIf(objc.arch == arch, f"{arch} only")(function)
+        return _unittest.skipUnless(objc.arch == arch, f"{arch} only")(function)
 
     return decorator
 
@@ -309,7 +213,9 @@ def min_python_release(version):
                 pass
     """
     parts = tuple(map(int, version.split(".")))
-    return onlyIf(_sys.version_info[:2] >= parts, f"Requires Python {version} or later")
+    return _unittest.skipUnless(
+        _sys.version_info[:2] >= parts, f"Requires Python {version} or later"
+    )
 
 
 def _sort_key(version):
@@ -340,7 +246,7 @@ def min_sdk_level(release):
                 pass
     """
     v = (objc.PyObjC_BUILD_RELEASE // 100, objc.PyObjC_BUILD_RELEASE % 100, 0)
-    return onlyIf(
+    return _unittest.skipUnless(
         v >= os_level_key(release), f"Requires build with SDK {release} or later"
     )
 
@@ -355,7 +261,7 @@ def max_sdk_level(release):
                 pass
     """
     v = (objc.PyObjC_BUILD_RELEASE // 100, objc.PyObjC_BUILD_RELEASE % 100, 0)
-    return onlyIf(
+    return _unittest.skipUnless(
         v <= os_level_key(release), f"Requires build with SDK {release} or later"
     )
 
@@ -370,7 +276,7 @@ def min_os_level(release):
             def testSnowLeopardCode(self):
                 pass
     """
-    return onlyIf(
+    return _unittest.skipUnless(
         os_level_key(os_release()) >= os_level_key(release),
         f"Requires OSX {release} or later",
     )
@@ -386,7 +292,7 @@ def max_os_level(release):
             def testUntilLeopard(self):
                 pass
     """
-    return onlyIf(
+    return _unittest.skipUnless(
         os_level_key(os_release()) <= os_level_key(release),
         f"Requires OSX up to {release}",
     )
@@ -402,7 +308,7 @@ def os_level_between(min_release, max_release):
             def testUntilLeopard(self):
                 pass
     """
-    return onlyIf(
+    return _unittest.skipUnless(
         os_level_key(min_release)
         <= os_level_key(os_release())
         <= os_level_key(max_release),
@@ -417,13 +323,8 @@ _nscftype = tuple(cls for cls in objc.getClassList() if "NSCFType" in cls.__name
 
 _typealias = {}
 
-if not is32Bit():
-    _typealias[objc._C_LNG_LNG] = objc._C_LNG
-    _typealias[objc._C_ULNG_LNG] = objc._C_ULNG
-
-else:  # pragma: no cover (32-bit)
-    _typealias[objc._C_LNG] = objc._C_INT
-    _typealias[objc._C_ULNG] = objc._C_UINT
+_typealias[objc._C_LNG_LNG] = objc._C_LNG
+_typealias[objc._C_ULNG_LNG] = objc._C_ULNG
 
 
 class TestCase(_unittest.TestCase):
@@ -444,272 +345,6 @@ class TestCase(_unittest.TestCase):
         if hasattr(func, "__metadata__"):
             self.fail(f"{func} has automatic bindings")
 
-    def assertCallable(self, func, *, argcount, variadic=None, deprecated=None):
-        """
-        Assert basic information about a method or function
-        """
-
-        results = []
-        if not hasattr(func, "__metadata__"):
-            self.fail("{func} has manual bindings")
-
-        info = func.__metadata__()
-
-        if isinstance(func, objc.selector):
-            cnt = info["arguments"] - 2
-        else:
-            cnt = info["arguments"]
-
-        if cnt != argcount:
-            results.append(f"{cnt} argument, expected {argcount}")
-
-        if variadic is not None:
-            if func.get("variadic") is not variadic:
-                results.append(f"variadic {info['variadic']!r} is not {variadic!r}")
-
-        else:
-            if info.get("variadic") is True:
-                results.append("variadic is True, not tested")
-
-        if deprecated is not None:
-            if info.get("deprecated") != deprecated:
-                results.append(f"variadic {info['deprecated']!r} != {deprecated!r}")
-
-        else:
-            if info.get("deprecated") is not None:
-                results.append("deprecated is set, not tested")
-
-        if results:
-            self.fail(f"{func}: " + ", ".join(results))
-
-    def assertReturns(
-        self,
-        func,
-        *,
-        type_encoding=None,
-        null_accepted=None,
-        already_retained=None,
-        already_cfretained=None,
-        callable=None,  # noqa: A002
-        c_array_of_variable_length=None,
-        c_array_delimited_by_null=None,
-    ):
-        """
-        Assert information about a return value
-        """
-
-        results = []
-        info = func.__metadata__()["retval"]
-
-        if type_encoding is not None:
-            if info["type"] != type_encoding:
-                results.append(f"encoding {info['type']!r} != {type_encoding!r}")
-
-        if null_accepted is not None:
-            if info["null_accepted"] is not null_accepted:
-                results.append(
-                    f"null_accepted {info['null_accepted']!r} is not {null_accepted!r}"
-                )
-
-        else:
-            if info["null_accepted"] is False:
-                results.append("null_accepted is False, not tested")
-
-        if already_retained is not None:
-            if info["already_retained"] is not already_retained:
-                results.append(
-                    f"already_retained {info['already_retained']!r} is not {already_retained!r}"
-                )
-
-        else:
-            if info["already_retained"] is True:
-                results.append("already_retained is True, not tested")
-
-        if already_cfretained is not None:
-            if info["already_cfretained"] is not already_cfretained:
-                results.append(
-                    f"already_cfretained {info['already_cfretained']!r} is not {already_cfretained!r}"
-                )
-
-        else:
-            if info["already_cfretained"] is True:
-                results.append("already_cfretained is True, not tested")
-
-        if c_array_of_variable_length is not None:
-            if info["c_array_of_variable_length"] is not c_array_of_variable_length:
-                results.append(
-                    f"c_array_of_variable_length {info['c_array_of_variable_length']!r} is not {c_array_of_variable_length!r}"
-                )
-
-        else:
-            if info["c_array_of_variable_length"] is True:
-                results.append("c_array_of_variable_length is True, not tested")
-
-        if c_array_delimited_by_null is not None:
-            if info["c_array_delimited_by_null"] is not c_array_delimited_by_null:
-                results.append(
-                    f"c_array_delimited_by_null {info['c_array_delimited_by_null']!r} is not {c_array_delimited_by_null!r}"
-                )
-
-        else:
-            if info["c_array_delimited_by_null"] is True:
-                results.append("c_array_delimited_by_null is True, not tested")
-
-        if callable is not None:
-            raise RuntimeError("Need to design API for this")
-
-        if results:
-            self.fail(f"{func} return-value: " + ", ".join(results))
-
-    def assertArgument(
-        self,
-        func,
-        argno,
-        *,
-        type_encoding=None,
-        type_modifier=None,
-        null_accepted=None,
-        printf_format=None,
-        already_retained=None,
-        already_cfretained=None,
-        callable=None,  # noqa: A002
-        callable_retaind=None,
-        c_array_length_in_arg=None,
-        c_array_length_in_result=None,
-        c_array_of_variable_length=None,
-        c_array_delimited_by_null=None,
-    ):
-        """
-        Assert information about an argument
-        """
-        try:
-            if isinstance(func, objc.selector):
-                info = func.__metadata__()["arguments"][argno + 2]
-            else:
-                info = func.__metadata__()["arguments"][argno]
-        except IndexError:
-            self.fail(f"{func.__name__!r} does not have argument {argno}")
-
-        results = []
-        have_modifier = False
-        have_array = False
-
-        if type_encoding is not None:
-            if type_modifier is not None:
-                expected_type = type_modifier + type_encoding
-
-            else:
-                expected_type = type_encoding
-
-            if info["type"] != expected_type:
-                results.append(f"encoding {info['type']!r} != {expected_type!r}")
-
-        elif type_modifier is not None:
-            if not info["type"].startswith(type_modifier + b"^") and not info[
-                "type"
-            ].startswith(type_modifier + b"*"):
-                results.append(
-                    f"modifier {type_modifier!r} not set or set for non-pointer {info['type']!r}"
-                )
-            else:
-                have_modifier = True
-
-        else:
-            if info["type"][0] in (objc._C_IN, objc._C_OUT, objc._C_INOUT):
-                results.append("modifier is set, not tested")
-                have_modifier = True
-
-        if null_accepted is not None:
-            if info["null_accepted"] is not null_accepted:
-                results.append(
-                    f"null_accepted {info['null_accepted']!r} is not {null_accepted!r}"
-                )
-
-        else:
-            if info["null_accepted"] is False:
-                results.append("null_accepted is False, not tested")
-
-        if printf_format is not None:
-            if info["printf_format"] is not null_accepted:
-                results.append(
-                    f"printf_format {info['printf_format']!r} is not {printf_format!r}"
-                )
-
-        else:
-            if info["printf_format"] is True:
-                results.append("printf_format is True, not tested")
-
-        if already_retained is not None:
-            if info["already_retained"] is not already_retained:
-                results.append(
-                    f"already_retained {info['already_retained']!r} is not {already_retained!r}"
-                )
-
-        else:
-            if info["already_retained"] is True:
-                results.append("already_retained is True, not tested")
-
-        if already_cfretained is not None:
-            if info["already_cfretained"] is not already_cfretained:
-                results.append(
-                    f"already_cfretained {info['already_cfretained']!r} is not {already_cfretained!r}"
-                )
-
-        else:
-            if info["already_cfretained"] is True:
-                results.append("already_cfretained is True, not tested")
-
-        if c_array_of_variable_length is not None:
-            have_array = True
-            if info["c_array_of_variable_length"] is not c_array_of_variable_length:
-                results.append(
-                    f"c_array_of_variable_length {info['c_array_of_variable_length']!r} is not {c_array_of_variable_length!r}"
-                )
-
-        if c_array_length_in_arg is not None:
-            have_array = True
-            if info["c_array_length_in_arg"] != c_array_length_in_arg:
-                results.append(
-                    f"c_array_length_in_arg {info['c_array_length_in_arg']!r} != {c_array_length_in_arg!r}"
-                )
-
-        else:
-            if info["c_array_length_in_arg"] is not None:
-                results.append("c_array_length_in_arg is set, not tested")
-
-        if c_array_length_in_result is not None:
-            have_array = True
-            if info["c_array_length_in_result"] is not c_array_length_in_result:
-                results.append(
-                    f"c_array_length_in_result {info['c_array_length_in_result']!r} is not {c_array_length_in_result!r}"
-                )
-
-        else:
-            if info["c_array_length_in_result"] is True:
-                results.append("c_array_length_in_arg is True, not tested")
-
-        if c_array_delimited_by_null is not None:
-            have_array = True
-            if info["c_array_delimited_by_null"] is not c_array_delimited_by_null:
-                results.append(
-                    f"c_array_delimited_by_null {info['c_array_delimited_by_null']!r} is not {c_array_delimited_by_null!r}"
-                )
-
-        else:
-            if info["c_array_delimited_by_null"] is True:
-                results.append("c_array_delimited_by_null is True, not tested")
-
-        if have_array and not have_modifier:
-            results.append("array, not no modifier specified")
-
-        if callable is not None:
-            raise RuntimeError("Need to design API for this")
-
-        if results:
-            self.fail(f"{func} arg {argno}: " + ", ".join(results))
-
-    # ----
-
     def assertIsCFType(self, tp, message=None):
         if not isinstance(tp, objc.objc_class):
             self.fail(message or f"{tp!r} is not a CFTypeRef type")
@@ -733,6 +368,27 @@ class TestCase(_unittest.TestCase):
         # if not issubclass(tp, _nscftype):
         #    self.fail(message or "%r is not a CFTypeRef subclass"%(tp,))
 
+    def assertIsEnumType(self, tp):
+        tmp = _typing.NewType("tmp", int)
+        if isinstance(tmp, type(lambda: 1)):
+            # typing.NewType is not a type in Python 3.7 or earlier
+            if not isinstance(tp, type(lambda: 1)):
+                self.fail(f"{tp!r} is not a typing.NewType")
+
+            if "NewType" not in str(tp) or "new_type" not in str(tp):
+                self.fail(f"{tp!r} is not a typing.NewType")
+
+            if tp.__supertype__ != int:
+                self.fail(f"{tp!r} is not a typing.NewType based on 'int'")
+
+            return
+
+        if not isinstance(tp, _typing.NewType):
+            self.fail(f"{tp!r} is not a typing.NewType")
+
+        if tp.__supertype__ != int:
+            self.fail(f"{tp!r} is not a typing.NewType based on 'int'")
+
     def assertIsOpaquePointer(self, tp, message=None):
         if not hasattr(tp, "__pointer__"):
             self.fail(message or f"{tp!r} is not an opaque-pointer")
@@ -846,9 +502,11 @@ class TestCase(_unittest.TestCase):
 
         try:
             if not info["arguments"][argno + offset]["already_cfretained"]:
-                self.fail(message or f"{method!r} is not cfretained")
+                self.fail(
+                    message or f"Argument {argno} of {method!r} is not cfretained"
+                )
         except (KeyError, IndexError):
-            self.fail(message or f"{method!r} is not cfretained")
+            self.fail(message or f"Argument {argno} of {method!r} is not cfretained")
 
     def assertArgIsNotCFRetained(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -858,7 +516,7 @@ class TestCase(_unittest.TestCase):
         info = method.__metadata__()
         try:
             if info["arguments"][argno + offset]["already_cfretained"]:
-                self.fail(message or f"{method!r} is cfretained")
+                self.fail(message or f"Argument {argno} of {method!r} is cfretained")
         except (KeyError, IndexError):
             pass
 
@@ -882,9 +540,9 @@ class TestCase(_unittest.TestCase):
 
         try:
             if not info["arguments"][argno + offset]["already_retained"]:
-                self.fail(message or f"{method!r} is not retained")
+                self.fail(message or f"Argument {argno} of {method!r} is not retained")
         except (KeyError, IndexError):
-            self.fail(message or f"{method!r} is not retained")
+            self.fail(message or f"Argument {argno} of {method!r} is not retained")
 
     def assertArgIsNotRetained(self, method, argno, message=None):
         if isinstance(method, objc.selector):
@@ -894,19 +552,19 @@ class TestCase(_unittest.TestCase):
         info = method.__metadata__()
         try:
             if info["arguments"][argno + offset]["already_retained"]:
-                self.fail(message or f"{method!r} is retained")
+                self.fail(message or f"Argument {argno} of {method!r} is retained")
         except (KeyError, IndexError):
             pass
 
     def assertResultIsRetained(self, method, message=None):
         info = method.__metadata__()
         if not info.get("retval", {}).get("already_retained", False):
-            self.fail(message or f"{method!r} is not retained")
+            self.fail(message or f"Result of {method!r} is not retained")
 
     def assertResultIsNotRetained(self, method, message=None):
         info = method.__metadata__()
         if info.get("retval", {}).get("already_retained", False):
-            self.fail(message or f"{method!r} is retained")
+            self.fail(message or f"Result of {method!r} is retained")
 
     def assertResultHasType(self, method, tp, message=None):
         info = method.__metadata__()
@@ -1317,106 +975,17 @@ class TestCase(_unittest.TestCase):
                 message or "arg %d of %s is not an 'in' argument" % (argno, method)
             )
 
-    #
-    # Addition assert methods, all of them should only be necessary for
-    # python 2.7 or later
-    #
+    def assertStartswith(self, value, test, message=None):  # pragma: no cover
+        if not value.startswith(test):
+            self.fail(message or f"{value!r} does not start with {test!r}")
 
-    if not hasattr(_unittest.TestCase, "assertItemsEqual"):  # pragma: no cover
+    def assertHasAttr(self, value, key, message=None):
+        if not hasattr(value, key):
+            self.fail(message or f"{key} is not an attribute of {value!r}")
 
-        def assertItemsEqual(self, seq1, seq2, message=None):
-            # This is based on unittest.util._count_diff_all_purpose from
-            # Python 2.7
-            s, t = list(seq1), list(seq2)
-            m, n = len(s), len(t)
-            NULL = object()
-            result = []
-            for i, elem in enumerate(s):
-                if elem is NULL:
-                    continue
-
-                cnt_s = cnt_t = 0
-                for j in range(i, m):
-                    if s[j] == elem:
-                        cnt_s += 1
-                        s[j] = NULL
-
-                for j, other_elem in enumerate(t):
-                    if other_elem == elem:
-                        cnt_t += 1
-                        t[j] = NULL
-
-                if cnt_s != cnt_t:
-                    result.append((cnt_s, cnt_t, elem))
-            for i, elem in enumerate(t):
-                if elem is NULL:
-                    continue
-                cnt_t = 0
-                for j in range(i, n):
-                    if t[j] == elem:
-                        cnt_t += 1
-                        t[j] = NULL
-
-                result.append((0, cnt_t, elem))
-
-            if result:
-                for actual, expected, value in result:
-                    print("Seq1 %d, Seq2: %d  value: %r" % (actual, expected, value))
-
-                self.fail(
-                    message
-                    or (
-                        "sequences do not contain the same items:"
-                        + "\n".join(
-                            ["Seq1 %d, Seq2: %d  value: %r" % (item) for item in result]
-                        )
-                    )
-                )
-
-    if not hasattr(_unittest.TestCase, "assertStartswith"):
-
-        def assertStartswith(self, value, test, message=None):  # pragma: no cover
-            if not value.startswith(test):
-                self.fail(message or f"{value!r} does not start with {test!r}")
-
-    if not hasattr(_unittest.TestCase, "assertIs"):  # pragma: no cover
-
-        def assertIs(self, value, test, message=None):
-            if value is not test:
-                self.fail(
-                    message
-                    or "%r (id=%r) is not %r (id=%r) "
-                    % (value, id(value), test, id(test))
-                )
-
-    if not hasattr(_unittest.TestCase, "assertIsNot"):  # pragma: no cover
-
-        def assertIsNot(self, value, test, message=None):
-            if value is test:
-                self.fail(message or f"{value!r} is {test!r}")
-
-    if not hasattr(_unittest.TestCase, "assertIsNone"):  # pragma: no cover
-
-        def assertIsNone(self, value, message=None):
-            self.assertIs(value, None)
-
-    if not hasattr(_unittest.TestCase, "assertIsNotNone"):  # pragma: no cover
-
-        def assertIsNotNone(self, value, message=None):
-            if value is None:
-                self.fail(message, f"{value!r} is not None")
-
-    if not hasattr(_unittest.TestCase, "assertHasAttr"):  # pragma: no cover
-
-        def assertHasAttr(self, value, key, message=None):
-            if not hasattr(value, key):
-                self.fail(message or f"{key} is not an attribute of {value!r}")
-
-    if not hasattr(_unittest.TestCase, "assertNotHasAttr"):  # pragma: no cover
-
-        def assertNotHasAttr(self, value, key, message=None):
-            if hasattr(value, key):
-                self.fail(message or f"{key} is an attribute of {value!r}")
+    def assertNotHasAttr(self, value, key, message=None):
+        if hasattr(value, key):
+            self.fail(message or f"{key} is an attribute of {value!r}")
 
     def assertIsSubclass(self, value, types, message=None):
         if not issubclass(value, types):
@@ -1426,71 +995,11 @@ class TestCase(_unittest.TestCase):
         if issubclass(value, types):
             self.fail(message or f"{value} is a subclass of {types!r}")
 
-    if not hasattr(_unittest.TestCase, "assertIsInstance"):  # pragma: no cover
-
-        def assertIsInstance(self, value, types, message=None):
-            if not isinstance(value, types):
-                self.fail(
-                    message
-                    or "%s is not an instance of %r but %s"
-                    % (value, types, type(value))
-                )
-
-    if not hasattr(_unittest.TestCase, "assertIsNotInstance"):  # pragma: no cover
-
-        def assertIsNotInstance(self, value, types, message=None):
-            if isinstance(value, types):
-                self.fail(message or f"{value} is an instance of {types!r}")
-
-    if not hasattr(_unittest.TestCase, "assertIn"):  # pragma: no cover
-
-        def assertIn(self, value, seq, message=None):
-            if value not in seq:
-                self.fail(message or f"{value!r} is not in {seq!r}")
-
-    if not hasattr(_unittest.TestCase, "assertNotIn"):  # pragma: no cover
-
-        def assertNotIn(self, value, seq, message=None):
-            if value in seq:
-                self.fail(message or f"{value!r} is in {seq!r}")
-
-    if not hasattr(_unittest.TestCase, "assertGreaterThan"):  # pragma: no cover
-
-        def assertGreaterThan(self, val, test, message=None):
-            if not (val > test):
-                self.fail(message or f"{val!r} <= {test!r}")
-
-    if not hasattr(_unittest.TestCase, "assertGreaterEqual"):  # pragma: no cover
-
-        def assertGreaterEqual(self, val, test, message=None):
-            if not (val >= test):
-                self.fail(message or f"{val!r} < {test!r}")
-
-    if not hasattr(_unittest.TestCase, "assertLessThan"):  # pragma: no cover
-
-        def assertLessThan(self, val, test, message=None):
-            if not (val < test):
-                self.fail(message or f"{val!r} >= {test!r}")
-
-    if not hasattr(_unittest.TestCase, "assertLessEqual"):  # pragma: no cover
-
-        def assertLessEqual(self, val, test, message=None):
-            if not (val <= test):
-                self.fail(message or f"{val!r} > {test!r}")
-
-    if not hasattr(_unittest.TestCase, "assertAlmostEquals"):  # pragma: no cover
-
-        def assertAlmostEquals(self, val1, val2, message=None):
-            self.assertTrue(
-                abs(val1 - val2) < 0.00001,
-                message or f"abs({val1!r} - {val2!r}) >= 0.00001",
-            )
-
     def assertClassIsFinal(self, cls):
         if not isinstance(cls, objc.objc_class):
             self.fail(f"{cls} is not an Objective-C class")
         elif not cls.__objc_final__:
-            self.fail(f"{cls} is not an final class")
+            self.fail(f"{cls} is not a final class")
 
     def assertProtocolExists(self, name):
         ok = True
@@ -1507,12 +1016,32 @@ class TestCase(_unittest.TestCase):
             # Should never happen
             self.fail(f"Protocol {name!r} is not a protocol, but {type(proto)}")
 
+    def assertPickleRoundTrips(self, value):
+        try:
+            buf = _pickle.dumps(value)
+            clone = _pickle.loads(buf)
+        except Exception:
+            self.fail(f"{value} cannot be pickled")
+
+        self.assertEqual(clone, value)
+        self.assertIsInstance(clone, type(value))
+
+    def __init__(self, methodName="runTest"):
+        super().__init__(methodName)
+
+        testMethod = getattr(self, methodName)
+
+        if getattr(testMethod, "_no_autorelease_pool", False):
+            self._skip_usepool = True
+        else:
+            self._skip_usepool = False
+
     def run(self, *args):
         """
         Run the test, same as unittest.TestCase.run, but every test is
         run with a fresh autorelease pool.
         """
-        if _usepool:
+        if _usepool and not self._skip_usepool:
             p = _poolclass.alloc().init()
         else:
             p = 1
@@ -1526,25 +1055,8 @@ class TestCase(_unittest.TestCase):
 
 
 main = _unittest.main
-
-if hasattr(_unittest, "expectedFailure"):
-    expectedFailure = _unittest.expectedFailure
-
-else:  # pragma: no cover (py2.6)
-
-    def expectedFailure(func):
-        def test(self):
-            try:
-                func(self)
-
-            except AssertionError:
-                return
-
-            self.fail("test unexpectedly passed")
-
-        test.__name__ = func.__name__
-
-        return test
+expectedFailure = _unittest.expectedFailure
+skipUnless = _unittest.skipUnless
 
 
 def expectedFailureIf(condition):
@@ -1552,3 +1064,8 @@ def expectedFailureIf(condition):
         return expectedFailure
     else:
         return lambda func: func
+
+
+def no_autorelease_pool(func):
+    func._no_autorelease_pool = True
+    return func
