@@ -5,9 +5,8 @@ NSData needs to be handles specially for correctness reasons,
 and is therefore in the core instead of the Foundation
 framework wrappers.
 """
-from objc._objc import registerMetaDataForSelector
 from objc._convenience import addConvenienceForClass
-import sys
+from objc._objc import registerMetaDataForSelector
 
 registerMetaDataForSelector(
     b"NSData",
@@ -33,18 +32,14 @@ def nsdata__str__(self):
         return str(b"")
     return str(self.bytes().tobytes())
 
+
 def nsdata__bytes__(self):
     return bytes(self.bytes())
 
 
-# XXX: These NSData helpers should use Cocoa method calls,
-#      instead of creating a memoryview/buffer object.
 def nsdata__getitem__(self, item):
     buff = self.bytes()
-    try:
-        return buff[item]
-    except TypeError:
-        return buff[:][item]
+    return buff[item]
 
 
 def nsmutabledata__setitem__(self, item, value):
