@@ -7,7 +7,24 @@
 
 NS_ASSUME_NONNULL_BEGIN
 
-static id (*bundleForClassIMP)(id, SEL, Class);
+static void
+nsmaptable_objc_retain(NSMapTable* table __attribute__((__unused__)), const void* datum)
+{
+    CFRetain((id)datum);
+}
+
+static void
+nsmaptable_objc_release(NSMapTable* table __attribute__((__unused__)), void* datum)
+{
+    CFRelease((id)datum);
+}
+
+static NSMapTableValueCallBacks PyObjC_ObjCValueCallBacks = {
+    &nsmaptable_objc_retain, &nsmaptable_objc_release,
+    NULL // generic description
+};
+
+static id (*bundleForClassIMP)(id, SEL, Class) = nil;
 
 @implementation OC_NSBundleHackCheck
 + (NSBundle* _Nullable)bundleForClass
@@ -29,8 +46,8 @@ static id (*bundleForClassIMP)(id, SEL, Class);
 
     if (unlikely(!bundleCache)) {
         bundleCache =
-            NSCreateMapTable(PyObjCUtil_PointerKeyCallBacks,
-                             PyObjCUtil_ObjCValueCallBacks, PYOBJC_EXPECTED_CLASS_COUNT);
+            NSCreateMapTable(PyObjCUtil_PointerKeyCallBacks, PyObjC_ObjCValueCallBacks,
+                             PYOBJC_EXPECTED_CLASS_COUNT);
     }
 
     if (!aClass) {
@@ -92,6 +109,12 @@ static const char BUNDLE_FOR_CLASS_SIGNATURE[] = {_C_ID, _C_ID, _C_SEL, _C_CLASS
                                  [self methodForSelector:@selector(bundleForClass:)]);
     }
 }
+
++ (BOOL)bundleHackUsed
+{
+    return (bundleForClassIMP != nil);
+}
+
 @end
 
 NS_ASSUME_NONNULL_END
