@@ -12,8 +12,8 @@ NS_ASSUME_NONNULL_BEGIN
 - (id _Nullable)initWithPythonObject:(PyObject*)v
 {
     self = [super init];
-    if (unlikely(self == nil))
-        return nil;
+    if (unlikely(self == nil)) // LCOV_BR_EXCL_LINE
+        return nil;            // LCOV_EXCL_LINE
 
     SET_FIELD_INCREF(value, v);
     return self;
@@ -31,6 +31,8 @@ NS_ASSUME_NONNULL_BEGIN
     /*
      * XXX: Check if 'value' can ever be NULL
      *      if not: replace by PyObjC_Assert check
+     *
+     * XXX: return [self __pyobjc_PythonObject__];
      */
     if (likely(value)) {
         Py_INCREF(value);
@@ -41,11 +43,6 @@ NS_ASSUME_NONNULL_BEGIN
     }
 }
 
-- (BOOL)supportsWeakPointers
-{
-    return YES;
-}
-
 + (BOOL)supportsSecureCoding
 {
     return NO;
@@ -54,22 +51,33 @@ NS_ASSUME_NONNULL_BEGIN
 - (oneway void)release
 {
     /* See comment in OC_PythonUnicode */
-    if (unlikely(!Py_IsInitialized())) {
+    if (unlikely(!Py_IsInitialized())) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [super release];
         return;
+        // LCOV_EXCL_STOP
     }
     PyObjC_BEGIN_WITH_GIL
 
-        [super release];
+        @try {
+            [super release];
+        } @catch (NSException* exc) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
+            PyObjC_LEAVE_GIL;
+            @throw;
+            // LCOV_EXCL_STOP
+        }
 
     PyObjC_END_WITH_GIL
 }
 
 - (void)dealloc
 {
-    if (unlikely(!Py_IsInitialized())) {
+    if (unlikely(!Py_IsInitialized())) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [super dealloc];
         return;
+        // LCOV_EXCL_STOP
     }
 
     PyObjC_BEGIN_WITH_GIL
@@ -167,7 +175,7 @@ NS_ASSUME_NONNULL_BEGIN
 
 - (void)getObjects:(id*)buffer inRange:(NSRange)range
 {
-    unsigned int i;
+    NSUInteger i;
 
     for (i = 0; i < range.length; i++) {
         buffer[i] = [self objectAtIndex:i + range.location];
@@ -179,9 +187,14 @@ NS_ASSUME_NONNULL_BEGIN
     PyObject* v;
     PyObject* w;
 
+    static NSObject* NSNull_null = nil;
+    if (NSNull_null == nil) {
+        NSNull_null = [NSNull null];
+    }
+
     PyObjC_BEGIN_WITH_GIL
 
-        if (unlikely(anObject == [NSNull null])) {
+        if (unlikely(anObject == NSNull_null)) {
             Py_INCREF(Py_None);
             v = Py_None;
         } else {
@@ -208,6 +221,11 @@ NS_ASSUME_NONNULL_BEGIN
     PyObject* v;
     PyObject* w;
 
+    static NSObject* NSNull_null = nil;
+    if (NSNull_null == nil) {
+        NSNull_null = [NSNull null];
+    }
+
     if (unlikely(idx > PY_SSIZE_T_MAX)) {
         PyObjC_BEGIN_WITH_GIL
             PyErr_SetString(PyExc_IndexError, "No such index");
@@ -216,7 +234,7 @@ NS_ASSUME_NONNULL_BEGIN
     }
 
     PyObjC_BEGIN_WITH_GIL
-        if (unlikely(anObject == [NSNull null])) {
+        if (unlikely(anObject == NSNull_null)) {
             Py_INCREF(Py_None);
             v = Py_None;
         } else {
@@ -228,8 +246,8 @@ NS_ASSUME_NONNULL_BEGIN
 
         PyObject* args[4] = {NULL, value, NULL, v};
         args[2]           = PyLong_FromUnsignedLong(idx);
-        if (args[2] == NULL) {
-            PyObjC_GIL_FORWARD_EXC();
+        if (args[2] == NULL) {        // LCOV_BR_EXCL_LINE
+            PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
         }
 
         w = PyObject_VectorcallMethod(PyObjCNM_insert, args + 1,
@@ -305,9 +323,14 @@ NS_ASSUME_NONNULL_BEGIN
         Py_ssize_t size = PyTuple_Size(value);
 
         if ([coder allowsKeyedCoding]) {
-            if (size > INT_MAX) {
+            if (size > INT_MAX) { // LCOV_BT_EXCL_LINE
+                /* Excluded from coverage tests because this would require creating
+                 * rather huge tuples, with corresponding memory usage.
+                 */
+                // LCOV_EXCL_START
                 [coder encodeInt32:5 forKey:@"pytype"];
                 [coder encodeInt64:(int64_t)PyTuple_Size(value) forKey:@"pylength"];
+                // LCOV_EXCL_STOP
 
             } else {
                 [coder encodeInt32:4 forKey:@"pytype"];
@@ -363,23 +386,29 @@ NS_ASSUME_NONNULL_BEGIN
      * protocol of NSArray.
      */
     NSUInteger i;
+
+    static NSObject* NSNull_null = nil;
+    if (NSNull_null == nil) {
+        NSNull_null = [NSNull null];
+    }
+
     PyObjC_BEGIN_WITH_GIL
         if (PyTuple_CheckExact(value) && (NSUInteger)PyTuple_Size(value) == count) {
             for (i = 0; i < count; i++) {
                 PyObject* v;
 
-                if (objects[i] == [NSNull null]) {
+                if (objects[i] == NSNull_null) {
                     v = Py_None;
                     Py_INCREF(Py_None);
 
                 } else {
                     v = id_to_python(objects[i]);
+                    if (v == NULL) {
+                        PyObjC_GIL_FORWARD_EXC();
+                    }
                 }
 
-                if (v == NULL) {
-                    PyObjC_GIL_FORWARD_EXC();
-                }
-
+                /* XXX: Can this every be true? */
                 if (PyTuple_GET_ITEM(value, i) != NULL) {
                     /* use temporary object to avoid race condition */
                     PyObject* t = PyTuple_GET_ITEM(value, i);
@@ -394,21 +423,20 @@ NS_ASSUME_NONNULL_BEGIN
                 PyObject* v;
                 int       r;
 
-                if (objects[i] == [NSNull null]) {
+                if (objects[i] == NSNull_null) {
                     v = Py_None;
                     Py_INCREF(Py_None);
 
                 } else {
                     v = id_to_python(objects[i]);
-                }
-
-                if (v == NULL) {
-                    PyObjC_GIL_FORWARD_EXC();
+                    if (v == NULL) {
+                        PyObjC_GIL_FORWARD_EXC();
+                    }
                 }
 
                 r = PyList_Append(value, v);
-                if (r == -1) {
-                    PyObjC_GIL_FORWARD_EXC();
+                if (r == -1) {                // LCOV_BR_EXCL_LINE
+                    PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
                 }
 
                 Py_DECREF(v);
@@ -426,6 +454,7 @@ NS_ASSUME_NONNULL_BEGIN
 - (void)pyobjcSetValue:(NSObject*)other
 {
     PyObjC_BEGIN_WITH_GIL
+        /* XXX: What if "other" cannot be proxied? */
         PyObject* v = id_to_python(other);
         SET_FIELD(value, v);
 
@@ -450,11 +479,15 @@ NS_ASSUME_NONNULL_BEGIN
     case 1:
         /* This code was used by some previous versions of PyObjC
          * (before 2.2) and is kept around for backward compatibility.
+         *
+         * Excluded from coverage testing because I don't have
+         * old archives that can be used for testing.
          */
+        // LCOV_EXCL_START
         PyObjC_BEGIN_WITH_GIL
             value = PyList_New(0);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (value == NULL) {          // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
 
         PyObjC_END_WITH_GIL
@@ -470,17 +503,18 @@ NS_ASSUME_NONNULL_BEGIN
             t     = value;
             value = PyList_AsTuple(t);
             Py_DECREF(t);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (value == NULL) {          // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
         PyObjC_END_WITH_GIL
         return self;
+        // LCOV_EXCL_STOP
 
     case 2:
         PyObjC_BEGIN_WITH_GIL
             value = PyList_New(0);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (value == NULL) {          // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
 
         PyObjC_END_WITH_GIL
@@ -492,8 +526,8 @@ NS_ASSUME_NONNULL_BEGIN
     case 3:
         PyObjC_BEGIN_WITH_GIL
             value = PyList_New(0);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (value == NULL) {          // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
 
         PyObjC_END_WITH_GIL
@@ -551,8 +585,8 @@ NS_ASSUME_NONNULL_BEGIN
 
         PyObjC_BEGIN_WITH_GIL
             value = PyTuple_New(size);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (value == NULL) {          // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
 
         PyObjC_END_WITH_GIL
@@ -563,6 +597,10 @@ NS_ASSUME_NONNULL_BEGIN
 
     case 5:
         /* tuple with more than MAX_INT elements */
+        /* Excluded from coverage check because testing this would
+         * require creating too large tuples, with corresponding memory use
+         */
+        // LCOV_BR_EXCL_START
         if ([coder allowsKeyedCoding]) {
             size = [coder decodeInt64ForKey:@"pylength"];
         } else {
@@ -571,13 +609,14 @@ NS_ASSUME_NONNULL_BEGIN
 
         PyObjC_BEGIN_WITH_GIL
             value = PyTuple_New(size);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (value == NULL) {          // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
 
         PyObjC_END_WITH_GIL
         tmpVal = [super initWithCoder:coder];
         PyObjC_Assert(tmpVal == self, nil);
+        // LCOV_EXCL_STOP
         return tmpVal;
 
     default:
@@ -591,6 +630,7 @@ NS_ASSUME_NONNULL_BEGIN
 - (id)copyWithZone:(NSZone* _Nullable)zone
 {
     if (PyObjC_CopyFunc) {
+        NSObject* result;
         PyObjC_BEGIN_WITH_GIL
 
             PyObject* copy = PyObjC_CallCopyFunc(value);
@@ -599,7 +639,6 @@ NS_ASSUME_NONNULL_BEGIN
                 PyObjC_GIL_FORWARD_EXC();
             }
 
-            NSObject* result;
             if (depythonify_python_object(copy, &result) == -1) {
                 Py_DECREF(copy);
                 PyObjC_GIL_FORWARD_EXC();
@@ -607,12 +646,11 @@ NS_ASSUME_NONNULL_BEGIN
 
             Py_DECREF(copy);
 
-            [result retain];
-
-            PyObjC_GIL_RETURN(result);
-
         PyObjC_END_WITH_GIL
 
+        [result retain];
+        return result;
+
     } else {
         return [super copyWithZone:zone];
     }
@@ -621,13 +659,13 @@ NS_ASSUME_NONNULL_BEGIN
 - (id)mutableCopyWithZone:(NSZone* _Nullable)zone
 {
     if (PyObjC_CopyFunc) {
+        NSObject* result;
         PyObjC_BEGIN_WITH_GIL
             PyObject* copy = PySequence_List(value);
             if (copy == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
             }
 
-            NSObject* result;
             if (depythonify_python_object(copy, &result) == -1) {
                 Py_DECREF(copy);
                 PyObjC_GIL_FORWARD_EXC();
@@ -635,10 +673,10 @@ NS_ASSUME_NONNULL_BEGIN
 
             Py_DECREF(copy);
 
-            [result retain];
-            PyObjC_GIL_RETURN(result);
-
         PyObjC_END_WITH_GIL
+
+        [result retain];
+        return result;
     } else {
         return [super mutableCopyWithZone:zone];
     }
