@@ -37,11 +37,6 @@ NS_ASSUME_NONNULL_BEGIN
     return value;
 }
 
-- (BOOL)supportsWeakPointers
-{
-    return YES;
-}
-
 + (BOOL)supportsSecureCoding
 {
     return NO;
@@ -56,7 +51,12 @@ NS_ASSUME_NONNULL_BEGIN
     }
 
     PyObjC_BEGIN_WITH_GIL
-        [super release];
+        @try {
+            [super release];
+        } @catch (NSException* exc) {
+            PyObjC_LEAVE_GIL;
+            [exc raise];
+        }
 
     PyObjC_END_WITH_GIL
 }
@@ -169,7 +169,12 @@ NS_ASSUME_NONNULL_BEGIN
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
             oc_value = [NSDate dateWithString:oc_v];
 #pragma clang diagnostic pop
-            [oc_value retain];
+            @try {
+                [oc_value retain];
+            } @catch (NSObject* exc) {
+                PyObjC_LEAVE_GIL;
+                @throw;
+            }
             Py_DECREF(v);
 
             if (oc_value == nil) {
@@ -213,9 +218,14 @@ NS_ASSUME_NONNULL_BEGIN
                 }
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
-                oc_value = [NSDate dateWithString:oc_v];
+                @try {
+                    oc_value = [NSDate dateWithString:oc_v];
 #pragma clang diagnostic pop
-                [oc_value retain];
+                    [oc_value retain];
+                } @catch (NSObject* exc) {
+                    PyObjC_LEAVE_GIL;
+                    @throw;
+                }
                 Py_DECREF(v);
             }
 
