@@ -96,11 +96,6 @@ NS_ASSUME_NONNULL_BEGIN
     return self;
 }
 
-- (BOOL)supportsWeakPointers
-{
-    return YES;
-}
-
 + (BOOL)supportsSecureCoding
 {
     return NO;
@@ -115,7 +110,12 @@ NS_ASSUME_NONNULL_BEGIN
     }
 
     PyObjC_BEGIN_WITH_GIL
-        [super release];
+        @try {
+            [super release];
+        } @catch (NSObject* exc) {
+            PyObjC_LEAVE_GIL;
+            @throw;
+        }
 
     PyObjC_END_WITH_GIL
 }
@@ -186,7 +186,7 @@ NS_ASSUME_NONNULL_BEGIN
 
     PyObjC_BEGIN_WITH_GIL
 
-        if (unlikely(key == [NSNull null])) {
+        if (unlikely(key == [NSNull null])) { /* XXX: NSNull_null */
             Py_INCREF(Py_None);
             k = Py_None;
         } else {
@@ -215,7 +215,7 @@ NS_ASSUME_NONNULL_BEGIN
         }
 
         if (v == Py_None) {
-            result = [NSNull null];
+            result = [NSNull null]; /* XXX: NSNull_null */
 
         } else if (unlikely(depythonify_python_object(v, &result) == -1)) {
             Py_DECREF(v);
@@ -232,7 +232,7 @@ NS_ASSUME_NONNULL_BEGIN
 {
     PyObject* v    = NULL;
     PyObject* k    = NULL;
-    id        null = [NSNull null];
+    id        null = [NSNull null]; /* XXX: NSNull_null */
 
     PyObjC_BEGIN_WITH_GIL
         if (unlikely(val == null)) {
@@ -283,7 +283,7 @@ NS_ASSUME_NONNULL_BEGIN
     PyObject* k;
 
     PyObjC_BEGIN_WITH_GIL
-        if (unlikely(key == [NSNull null])) {
+        if (unlikely(key == [NSNull null])) { /* XXX: NSNull_null */
             Py_INCREF(Py_None);
             k = Py_None;
         } else {
@@ -354,7 +354,7 @@ NS_ASSUME_NONNULL_BEGIN
             PyObject* v;
             int       r;
 
-            if (objects[i] == [NSNull null]) {
+            if (objects[i] == [NSNull null]) { /* XXX: NSNull_null */
                 v = Py_None;
                 Py_INCREF(Py_None);
 
@@ -535,9 +535,9 @@ NS_ASSUME_NONNULL_BEGIN
                 PyObjC_GIL_FORWARD_EXC();
             }
             Py_DECREF(copy);
-            [result retain];
         PyObjC_END_WITH_GIL
 
+        [result retain];
         return result;
 
     } else {
@@ -567,10 +567,9 @@ NS_ASSUME_NONNULL_BEGIN
             }
             Py_DECREF(copy);
 
-            [result retain];
-
         PyObjC_END_WITH_GIL
 
+        [result retain];
         return result;
 
     } else {
