@@ -9,6 +9,9 @@
 
 #include <fcntl.h>
 
+// LCOV_BR_EXCL_START
+// LCOV_EXCL_START
+
 NS_ASSUME_NONNULL_BEGIN
 
 struct Struct1 {
@@ -843,12 +846,23 @@ ASSERT(v);
 
 buf = [[OCReleasedBuffer alloc] initWithPythonBuffer:v writable:NO];
 ASSERT(buf);
-ASSERT([buf buffer]);
-ASSERT(strncmp([buf buffer], "hello", 5) == 0);
+if (![buf buffer]) {
+    [buf release];
+    ASSERT(0);
+}
+
+if (strncmp([buf buffer], "hello", 5) != 0) {
+    [buf release];
+    ASSERT(0);
+}
+
 [buf release];
 
 buf = [[OCReleasedBuffer alloc] initWithPythonBuffer:v writable:YES];
-ASSERT(!buf);
+if (buf) {
+    [buf release];
+    ASSERT(!buf);
+}
 ASSERT(PyErr_Occurred());
 PyErr_Clear();
 
@@ -856,14 +870,32 @@ v = PyByteArray_FromStringAndSize("hello", 5);
 ASSERT(v);
 buf = [[OCReleasedBuffer alloc] initWithPythonBuffer:v writable:NO];
 ASSERT(buf);
-ASSERT([buf buffer]);
-ASSERT(strncmp([buf buffer], "hello", 5) == 0);
+if (![buf buffer]) {
+    Py_CLEAR(v);
+    [buf release];
+    ASSERT(0);
+}
+if (strncmp([buf buffer], "hello", 5) != 0) {
+    Py_CLEAR(v);
+    [buf release];
+    ASSERT(0);
+}
+
 [buf release];
 
 buf = [[OCReleasedBuffer alloc] initWithPythonBuffer:v writable:YES];
 ASSERT(buf);
-ASSERT([buf buffer]);
-ASSERT(strncmp([buf buffer], "hello", 5) == 0);
+if (![buf buffer]) {
+    Py_CLEAR(v);
+    [buf release];
+    ASSERT(0);
+}
+if (strncmp([buf buffer], "hello", 5) != 0) {
+    Py_CLEAR(v);
+    [buf release];
+    ASSERT(0);
+}
+Py_CLEAR(v);
 [buf release];
 
 END_UNITTEST
@@ -872,6 +904,35 @@ BEGIN_UNITTEST(AlwaysFails)
 ASSERT(0 == 1);
 END_UNITTEST
 
+BEGIN_UNITTEST(InvalidObjCPointer)
+
+/* Invalid type encoding */
+PyObject* p = PyObjCPointer_New(&p, "^{foo=");
+ASSERT(PyErr_Occurred());
+ASSERT(!p);
+PyErr_Clear();
+
+p = PyObjCPointer_New(&p, "{foo=");
+ASSERT(PyErr_Occurred());
+ASSERT(!p);
+PyErr_Clear();
+
+END_UNITTEST
+
+BEGIN_UNITTEST(InvalidRegistryUsage)
+PyObject* bytes = PyBytes_FromString("hello");
+if (bytes == NULL) {
+    return NULL;
+}
+
+int r = PyObjC_AddToRegistry(bytes, bytes, bytes, bytes);
+Py_DECREF(bytes);
+ASSERT(r == -1);
+
+PyErr_Clear();
+
+END_UNITTEST
+
 static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
 
                                     TESTDEF(StructSize),
@@ -904,6 +965,8 @@ static PyMethodDef mod_methods[] = {TESTDEF(CheckNSInvoke),
                                     TESTDEF(MemView),
                                     TESTDEF(ReleasedBuffer),
                                     TESTDEF(AlwaysFails),
+                                    TESTDEF(InvalidObjCPointer),
+                                    TESTDEF(InvalidRegistryUsage),
                                     {0, 0, 0, 0}};
 
 int
@@ -932,4 +995,7 @@ PyObjC_init_ctests(PyObject* m)
     return PyModule_AddObject(m, "_ctests", d);
 }
 
+// LCOV_BR_EXCL_STOP
+// LCOV_EXCL_STOP
+
 NS_ASSUME_NONNULL_END
