@@ -6,16 +6,18 @@
  */
 #include "pyobjc.h"
 
-static PyObject*
-find_selector(PyObject* self, const char* name, int class_method)
+NS_ASSUME_NONNULL_BEGIN
+
+static PyObject* _Nullable find_selector(PyObject* self, const char* name,
+                                         int class_method)
 {
-    SEL sel = PyObjCSelector_DefaultSelector(name);
-    id objc_object;
+    SEL                sel = PyObjCSelector_DefaultSelector(name);
+    id                 objc_object;
     NSMethodSignature* methsig;
-    char buf[1024];
-    int unbound_instance_method = 0;
-    char* flattened = NULL;
-    PyObject* class_object;
+    char               buf[1024];
+    int                unbound_instance_method = 0;
+    char*              flattened               = NULL;
+    PyObject*          class_object;
 
     if (name[0] == '_' && name[1] == '_') {
         /* There are no public methods that start with a double underscore,
@@ -26,7 +28,7 @@ find_selector(PyObject* self, const char* name, int class_method)
     }
 
     if (PyObjCClass_Check(self)) {
-        objc_object = (id)PyObjCClass_GetClass(self);
+        objc_object  = (id)PyObjCClass_GetClass(self);
         class_object = self;
 
         if (!class_method) {
@@ -60,7 +62,8 @@ find_selector(PyObject* self, const char* name, int class_method)
     }
 
     if (strcmp(object_getClassName(objc_object), "_NSZombie") == 0) {
-        PyErr_Format(PyExc_AttributeError, "Cannot access NSProxy.%s", name);
+        PyErr_Format(PyExc_AttributeError, "Cannot access '%s' on deallocated object",
+                     name);
         return NULL;
     }
 
@@ -93,6 +96,7 @@ find_selector(PyObject* self, const char* name, int class_method)
         objc_object = (id)object_getClass(objc_object);
     }
 
+    /* XXX: This needs documentation */
     PyObject* meta = PyObjCClass_HiddenSelector(class_object, sel, class_method);
 
     if (meta && meta != Py_None) {
@@ -110,16 +114,15 @@ find_selector(PyObject* self, const char* name, int class_method)
     return PyObjCSelector_NewNative((Class)objc_object, sel, flattened, class_method);
 }
 
-static PyObject*
-make_dict(PyObject* self, int class_method)
+static PyObject* _Nullable make_dict(PyObject* self, int class_method)
 {
-    Class cls;
-    PyObject* res;
-    Method* methods;
+    Class        cls;
+    PyObject*    res;
+    Method*      methods;
     unsigned int i, method_count;
-    void* iterator;
-    char buf[256];
-    Class objc_class;
+    void*        iterator;
+    char         buf[256];
+    Class        objc_class;
 
     if (PyObjCObject_Check(self)) {
         id obj = PyObjCObject_GetObject(self);
@@ -129,16 +132,16 @@ make_dict(PyObject* self, int class_method)
         }
 
         if (class_method) {
-            cls = object_getClass(obj);
+            cls        = object_getClass(obj);
             objc_class = object_getClass(cls);
 
         } else {
-            cls = object_getClass(obj);
+            cls        = object_getClass(obj);
             objc_class = cls;
         }
 
     } else if (PyObjCClass_Check(self)) {
-        cls = PyObjCClass_GetClass(self);
+        cls        = PyObjCClass_GetClass(self);
         objc_class = cls;
 
         if (class_method) {
@@ -146,6 +149,8 @@ make_dict(PyObject* self, int class_method)
         }
 
     } else {
+        /* XXX: Don't particularly like PyErr_BadInternalCall */
+        /* XXX: Verify if this can be triggered, if not: use PyObjC_InternalError() */
         PyErr_BadInternalCall();
         return NULL;
     }
@@ -157,19 +162,27 @@ make_dict(PyObject* self, int class_method)
 
     while (objc_class != NULL && cls != NULL) {
         iterator = NULL;
-        methods = class_copyMethodList(objc_class, &method_count);
+        methods  = class_copyMethodList(objc_class, &method_count);
 
         if (methods == NULL) {
+            /* XXX: this is the same as the code after the for loop below,
+             *      restructure the code.
+             */
             objc_class = class_getSuperclass((Class)objc_class);
-            cls = class_getSuperclass((Class)cls);
+            cls        = class_getSuperclass((Class)cls);
             continue;
         }
 
         for (i = 0; i < method_count; i++) {
             PyObject* v;
-            char* name;
+            char*     name;
 
             name = PyObjC_SELToPythonName(method_getName(methods[i]), buf, sizeof(buf));
+            if (name == NULL) {
+                free(methods);
+                Py_DECREF(res);
+                return NULL;
+            }
 
             v = PyObject_GetAttrString(self, name);
 
@@ -190,9 +203,17 @@ make_dict(PyObject* self, int class_method)
             }
 
             if (v == NULL) {
+                const char* type_encoding = method_getTypeEncoding(methods[i]);
+                if (type_encoding == NULL) {
+                    PyErr_SetString(PyObjCExc_Error,
+                                    "Native selector with Nil type encoding");
+                    free(methods);
+                    Py_DECREF(res);
+                    return NULL;
+                }
+
                 v = PyObjCSelector_NewNative(cls, method_getName(methods[i]),
-                                             method_getTypeEncoding(methods[i]),
-                                             class_method);
+                                             type_encoding, class_method);
 
                 if (v == NULL) {
                     free(methods);
@@ -214,24 +235,27 @@ make_dict(PyObject* self, int class_method)
         free(methods);
 
         objc_class = class_getSuperclass((Class)objc_class);
-        cls = class_getSuperclass((Class)cls);
+        cls        = class_getSuperclass((Class)cls);
     }
 
     return res;
 }
 
 typedef struct {
-    PyObject_HEAD PyObject* base;
-    int class_method;
+    PyObject_HEAD
+    PyObject* base;
+    int       class_method;
 } ObjCMethodAccessor;
 
 static void
 obj_dealloc(PyObject* _self)
 {
     ObjCMethodAccessor* self = (ObjCMethodAccessor*)_self;
-    Py_XDECREF(self->base);
-    self->base = NULL;
 
+    /* XXX: Can 'base' every be NULL? */
+    Py_XDECREF(self->base);
+
+    /* XXX:  Why not just call PyObject_Del? */
     if (Py_TYPE(self)->tp_free) {
         Py_TYPE(self)->tp_free((PyObject*)self);
 
@@ -240,11 +264,10 @@ obj_dealloc(PyObject* _self)
     }
 }
 
-static PyObject*
-obj_getattro(PyObject* _self, PyObject* name)
+static PyObject* _Nullable obj_getattro(PyObject* _self, PyObject* name)
 {
-    ObjCMethodAccessor* self = (ObjCMethodAccessor*)_self;
-    PyObject* result = NULL;
+    ObjCMethodAccessor* self   = (ObjCMethodAccessor*)_self;
+    PyObject*           result = NULL;
 
     if (PyUnicode_Check(name)) {
         if (PyObjC_Unicode_Fast_Bytes(name) == NULL) {
@@ -256,23 +279,26 @@ obj_getattro(PyObject* _self, PyObject* name)
         return NULL;
     }
 
-    if (strcmp(PyObjC_Unicode_Fast_Bytes(name), "__dict__") == 0) {
+    if (PyObjC_is_ascii_string(name, "__dict__")) {
 
         PyObject* dict;
         dict = make_dict(self->base, self->class_method);
+        if (dict == NULL) {
+            return NULL;
+        }
 
         result = PyDictProxy_New(dict);
         Py_DECREF(dict);
         return result;
     }
 
-    if (strcmp(PyObjC_Unicode_Fast_Bytes(name), "__methods__") == 0) {
+    if (PyObjC_is_ascii_string(name, "__methods__")) {
 
         PyErr_SetString(PyExc_AttributeError, "No such attribute: __methods__");
         return NULL;
     }
 
-    if (strcmp(PyObjC_Unicode_Fast_Bytes(name), "__members__") == 0) {
+    if (PyObjC_is_ascii_string(name, "__members__")) {
 
         PyErr_SetString(PyExc_AttributeError, "No such attribute: __members__");
         return NULL;
@@ -293,10 +319,10 @@ obj_getattro(PyObject* _self, PyObject* name)
             PyObject* descr_arg;
 
             if (PyObjCClass_Check(self->base)) {
-                mro = ((PyTypeObject*)self->base)->tp_mro;
+                mro       = ((PyTypeObject*)self->base)->tp_mro;
                 descr_arg = NULL;
             } else {
-                mro = (Py_TYPE(self->base))->tp_mro;
+                mro       = (Py_TYPE(self->base))->tp_mro;
                 descr_arg = self->base;
             }
             Py_ssize_t i, len;
@@ -308,7 +334,7 @@ obj_getattro(PyObject* _self, PyObject* name)
                     continue;
 
                 PyObject* dict = ((PyTypeObject*)c)->tp_dict;
-                PyObject* v = PyDict_GetItemWithError(dict, name);
+                PyObject* v    = PyDict_GetItemWithError(dict, name);
                 if (v == NULL && PyErr_Occurred()) {
                     return NULL;
 
@@ -360,8 +386,11 @@ obj_getattro(PyObject* _self, PyObject* name)
     }
 
     /* Didn't find the selector the first trip around, try harder. */
-    result =
-        find_selector(self->base, PyObjC_Unicode_Fast_Bytes(name), self->class_method);
+    const char* name_bytes = PyObjC_Unicode_Fast_Bytes(name);
+    if (name_bytes == NULL) {
+        return NULL;
+    }
+    result = find_selector(self->base, name_bytes, self->class_method);
     if (result == NULL) {
         return result;
     }
@@ -382,11 +411,10 @@ obj_getattro(PyObject* _self, PyObject* name)
     return result;
 }
 
-static PyObject*
-obj_repr(PyObject* _self)
+static PyObject* _Nullable obj_repr(PyObject* _self)
 {
     ObjCMethodAccessor* self = (ObjCMethodAccessor*)_self;
-    PyObject* rval;
+    PyObject*           rval;
 
     rval = PyUnicode_FromFormat("<%s method-accessor for %R>",
                                 self->class_method ? "class" : "instance", self->base);
@@ -394,8 +422,7 @@ obj_repr(PyObject* _self)
     return rval;
 }
 
-static PyObject*
-obj_dir(PyObject* self)
+static PyObject* _Nullable obj_dir(PyObject* self)
 {
     PyObject* dict = make_dict(((ObjCMethodAccessor*)self)->base,
                                ((ObjCMethodAccessor*)self)->class_method);
@@ -412,8 +439,8 @@ obj_dir(PyObject* self)
 }
 
 static PyMethodDef obj_methods[] = {{
-                                        .ml_name = "__dir__",
-                                        .ml_meth = (PyCFunction)obj_dir,
+                                        .ml_name  = "__dir__",
+                                        .ml_meth  = (PyCFunction)obj_dir,
                                         .ml_flags = METH_NOARGS,
                                     },
                                     {
@@ -422,20 +449,21 @@ static PyMethodDef obj_methods[] = {{
 
 PyTypeObject PyObjCMethodAccessor_Type = {
     PyVarObject_HEAD_INIT(&PyType_Type, 0).tp_name = "objc.method_acces",
-    .tp_basicsize = sizeof(ObjCMethodAccessor),
-    .tp_itemsize = 0,
-    .tp_dealloc = obj_dealloc,
-    .tp_repr = obj_repr,
-    .tp_getattro = obj_getattro,
-    .tp_flags = Py_TPFLAGS_DEFAULT,
-    .tp_methods = obj_methods,
+    .tp_basicsize                                  = sizeof(ObjCMethodAccessor),
+    .tp_itemsize                                   = 0,
+    .tp_dealloc                                    = obj_dealloc,
+    .tp_repr                                       = obj_repr,
+    .tp_getattro                                   = obj_getattro,
+    .tp_flags                                      = Py_TPFLAGS_DEFAULT,
+    .tp_methods                                    = obj_methods,
 };
 
-PyObject*
-PyObjCMethodAccessor_New(PyObject* base, int class_method)
+PyObject* _Nullable PyObjCMethodAccessor_New(PyObject* base, int class_method)
 {
     ObjCMethodAccessor* result;
 
+    /* XXX: This shou;ld check that 'base' is valid */
+
     result = PyObject_New(ObjCMethodAccessor, &PyObjCMethodAccessor_Type);
     if (result == NULL)
         return NULL;
@@ -446,3 +474,5 @@ PyObjCMethodAccessor_New(PyObject* base, int class_method)
 
     return (PyObject*)result;
 }
+
+NS_ASSUME_NONNULL_END
