@@ -207,6 +207,11 @@ determine_if_shortcut(PyObjCMethodSignature* methinfo)
     if (methinfo == NULL || methinfo->variadic) {
         return 0;
     }
+
+    if (methinfo->suggestion != NULL) {
+        return 0;
+    }
+
 #ifdef PyObjC_DEBUG
     if (PyObjCMethodSignature_Validate(methinfo) == -1)
         return -1;
@@ -590,6 +595,9 @@ char* _Nullable PyObjC_NSMethodSignatureToTypeString(NSMethodSignature* sig, cha
  *  0: OK
  * -1: error
  * -2: 'descr' is template, but would have to be updated.
+ *
+ *  XXX: Use warnings to warn about invalid metadata, with a specific warning that
+ *       can be easility turned into an error.
  */
 static int
 setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_native)
@@ -731,7 +739,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
             /* Make up a dummy signature, will be overridden by
              * the metadata.
              */
-            char      buffer[64];
+            char      buffer[128];
             PyObject* a = PyDict_GetItemStringWithError(d, "arguments");
             if (a == NULL && PyErr_Occurred()) {
                 return -1;
@@ -740,6 +748,11 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
                 if (len == -1) {
                     return -1;
                 }
+                if ((size_t)len >= sizeof(buffer) - 2) {
+                    PyErr_SetString(PyObjCExc_Error,
+                                    "Callable metadata with too many arguments");
+                    return -1;
+                }
 
                 for (i = 0; i < len; i++) {
                     buffer[i] = _C_ID;
@@ -946,7 +959,7 @@ setup_descr(struct _PyObjC_ArgDescr* descr, PyObject* _Nullable meta, BOOL is_na
             Py_INCREF(bytes);
 
         } else {
-            PyErr_SetString(PyExc_SystemError, "Inconsistent if-case");
+            PyObjC_Assert(0, -1);
             return -1;
         }
 
@@ -1032,7 +1045,10 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
     PyObject* v;
 
     if (metadata != NULL && !PyDict_Check(metadata)) {
-        metadata = NULL;
+        PyErr_Format(PyExc_TypeError,
+                     "Metadata dictionary is of type '%s' instead of 'dict'",
+                     Py_TYPE(metadata)->tp_name);
+        return -1;
     }
 
     if (metadata) {
@@ -1042,18 +1058,15 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
         } else if (retval != NULL) {
             int r = setup_descr(methinfo->rettype, retval, is_native);
             if (r == -1) {
-                Py_DECREF(methinfo);
                 return -1;
 
             } else if (r == -2) {
                 methinfo->rettype = alloc_descr(methinfo->rettype);
                 if (methinfo->rettype == NULL) {
-                    Py_DECREF(methinfo);
                     return -1;
                 }
                 r = setup_descr(methinfo->rettype, retval, is_native);
                 if (r == -1) {
-                    Py_DECREF(methinfo);
                     return -1;
                 }
                 PyObjC_Assert(r != -2, -1);
@@ -1100,11 +1113,11 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
                     continue;
                 }
 
-                PyObjC_Assert(
-                    methinfo->argtype[i] == NULL || methinfo->argtype[i]->allowNULL, -1);
+                // PyObjC_Assert(
+                //     methinfo->argtype[i] == NULL || methinfo->argtype[i]->allowNULL,
+                //     -1);
                 r = setup_descr(methinfo->argtype[i], d, is_native);
                 if (r == -1) {
-                    Py_DECREF(methinfo);
                     return -1;
 
                 } else if (r == -2) {
@@ -1115,7 +1128,6 @@ process_metadata_dict(PyObjCMethodSignature* methinfo, PyObject* _Nullable metad
                     }
                     r = setup_descr(methinfo->argtype[i], d, is_native);
                     if (r == -1) {
-                        Py_DECREF(methinfo);
                         return -1;
                     }
                     PyObjC_Assert(r != -2, -1);
@@ -1277,12 +1289,18 @@ PyObjC_registerMetaData(PyObject* class_name, PyObject* selector, PyObject* meta
 {
     PyObject* compiled;
     int       r;
+
+    /* XXX: Move creation of the registry to a separate function called from
+     * module init.
+     */
     if (registry == NULL) {
         registry = PyObjC_NewRegistry();
         if (registry == NULL) {
             return -1;
         }
     }
+    PyObjC_Assert(PyBytes_Check(class_name), -1);
+    PyObjC_Assert(PyBytes_Check(selector), -1);
     if (!PyDict_Check(metadata)) {
         PyErr_SetString(PyExc_TypeError, "metadata should be a dictionary");
         return -1;
@@ -1299,6 +1317,8 @@ PyObjC_registerMetaData(PyObject* class_name, PyObject* selector, PyObject* meta
      * Leak a reference to 'compiled' to ensure it stays alive
      * even when someone registers new metadata for the same
      * selector.
+     *
+     * XXX: Why is this needed?
      * -- Py_DECREF(compiled); --
      */
     return r;
@@ -1785,7 +1805,7 @@ PyObject* _Nullable PyObjCMethodSignature_AsDict(PyObjCMethodSignature* methinfo
     }
 
     if (methinfo->deprecated != 0) {
-        v = Py_BuildValue("i", &methinfo->deprecated);
+        v = PyLong_FromLong(methinfo->deprecated);
         if (v == NULL)
             goto error;
 
