@@ -27,7 +27,12 @@ static PyObject* _Nullable object_new(PyTypeObject* type __attribute__((__unused
         return NULL;
     }
 
-    if (cobject != NULL && PyCapsule_CheckExact(cobject)) {
+    if (cobject != NULL) {
+        if (!PyCapsule_CheckExact(cobject)) {
+            PyErr_SetString(PyExc_TypeError, "cobject' argument is not a PyCapsule");
+            return NULL;
+        }
+
         NSObject* p = PyCapsule_GetPointer(cobject, "objc.__object__");
         if (PyErr_Occurred()) {
             return NULL;
@@ -95,6 +100,10 @@ static PyObject* _Nullable object_repr(PyObject* _self)
                                                       1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         if (res == NULL) {
             PyErr_Clear();
+            /* Fall though to default */
+        } else if (res == Py_None) {
+            Py_DECREF(res);
+            /* Fall though to default */
         } else {
             return res;
         }
@@ -158,12 +167,7 @@ object_dealloc(PyObject* obj)
         } else {
             Py_BEGIN_ALLOW_THREADS
                 @try {
-                    /* XXX: Both branches are the same */
-                    if (((PyObjCObject*)obj)->flags & PyObjCObject_kCFOBJECT) {
-                        CFRelease(((PyObjCObject*)obj)->objc_object);
-                    } else {
-                        CFRelease(((PyObjCObject*)obj)->objc_object);
-                    }
+                    CFRelease(((PyObjCObject*)obj)->objc_object);
 
                 } @catch (NSObject* localException) {
                     NSLog(@"PyObjC: Exception during dealloc of proxy: %@",
@@ -1082,7 +1086,7 @@ PyObjCClassObject PyObjCObject_Type = {
     .base = {
         .ht_type =
             {
-                PyVarObject_HEAD_INIT(&PyObjCClass_Type, 0).tp_name = "objc_object",
+                PyVarObject_HEAD_INIT(&PyObjCClass_Type, 0).tp_name = "objc.objc_object",
                 .tp_basicsize = sizeof(PyObjCObject),
                 .tp_itemsize  = 0,
                 .tp_dealloc   = object_dealloc,
