@@ -449,7 +449,7 @@ const char* _Nullable PyObjCRT_SkipTypeSpec(const char* start_type)
                     type++;
                 } else {
                     PyErr_Format(PyObjCExc_InternalError,
-                                 "Invalid union definition in type signature: %s",
+                                 "Invalid union definition in type signature: '%s'",
                                  start_type);
                     return NULL;
                 }
@@ -458,7 +458,7 @@ const char* _Nullable PyObjCRT_SkipTypeSpec(const char* start_type)
         }
         if (type && *type != _C_UNION_E) {
             PyErr_Format(PyObjCExc_InternalError,
-                         "Invalid union definition in type signature: %s", start_type);
+                         "Invalid union definition in type signature: '%s'", start_type);
             return NULL;
         }
         if (type)
@@ -473,6 +473,11 @@ const char* _Nullable PyObjCRT_SkipTypeSpec(const char* start_type)
     case _C_BYCOPY:
     case _C_BYREF:
     case _C_ONEWAY:
+        if (type[1] == '\0') {
+            PyErr_Format(PyObjCExc_InternalError, "Incomplete type signature: '%s'",
+                         start_type);
+            return NULL;
+        }
 
         /* Just skip the following typespec */
         type = PyObjCRT_SkipTypeSpec(type + 1);
@@ -667,17 +672,12 @@ const char* _Nullable PyObjCRT_NextField(const char* start_type)
 
 /*
 Return the alignment of an object specified by type
+
+XXX: The embedded align function is used for the first
+member of a struct, and is no longer necessary.
 */
 
-static inline Py_ssize_t
-PyObjC_EmbeddedAlignOfType(const char* type)
-{
-    PyObjC_Assert(type != NULL, -1);
-
-    Py_ssize_t align = PyObjCRT_AlignOfType(type);
-
-    return align;
-}
+#define PyObjC_EmbeddedAlignOfType PyObjCRT_AlignOfType
 
 Py_ssize_t
 PyObjCRT_AlignOfType(const char* start_type)
@@ -1233,7 +1233,7 @@ static PyObject* _Nullable pythonify_c_struct(const char* type, void* datum)
     int         haveTuple;
     const char* type_start = type;
     const char* type_end   = PyObjCRT_SkipTypeSpec(type);
-    Py_ssize_t  pack;
+    Py_ssize_t  pack       = -1;
 
     if (type_end == NULL) {
         return NULL;
@@ -1249,9 +1249,6 @@ static PyObject* _Nullable pythonify_c_struct(const char* type, void* datum)
         return PyObjC_decode_fsref(datum);
     }
 
-    if (IS_FSSPEC(type)) {
-        return PyObjC_decode_fsspec(datum);
-    }
     if (IS_DECIMAL(type)) {
         return pythonify_nsdecimal(datum);
     }
@@ -1695,17 +1692,7 @@ depythonify_c_struct(const char* types, PyObject* arg, void* datum)
     }
 
     if (IS_FSREF(types)) {
-        if (PyObjC_encode_fsref(arg, datum) == 0) {
-            return 0;
-        }
-        PyErr_Clear();
-    }
-
-    if (IS_FSSPEC(types)) {
-        if (PyObjC_encode_fsspec(arg, datum) == 0) {
-            return 0;
-        }
-        PyErr_Clear();
+        return PyObjC_encode_fsref(arg, datum);
     }
 
     if (IS_DECIMAL(types)) {
@@ -2217,6 +2204,9 @@ depythonify_python_object(PyObject* argument, id* datum)
             Py_DECREF(anObject);
         }
     } else {
+        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
+            return -1;
+        }
         PyErr_Clear();
     }
 
@@ -2735,7 +2725,7 @@ depythonify_c_value(const char* type, PyObject* argument, void* datum)
                    PyByteArray_Check(argument) ||
 #endif
                    PyUnicode_Check(argument)) {
-            PyErr_Format(PyExc_ValueError, "depythonifying 'float', got '%s'",
+            PyErr_Format(PyExc_ValueError, "depythonifying 'double', got '%s'",
                          Py_TYPE(argument)->tp_name);
             return -1;
 
@@ -2900,8 +2890,10 @@ void
 PyObjCObject_ReleaseTransient(PyObject* proxy, int cookie)
 {
     if (cookie && Py_REFCNT(proxy) != 1) {
-        CFRetain(PyObjCObject_GetObject(proxy));
-        ((PyObjCObject*)proxy)->flags &= ~PyObjCObject_kSHOULD_NOT_RELEASE;
+        Py_BEGIN_ALLOW_THREADS
+            CFRetain(PyObjCObject_GetObject(proxy));
+            Py_END_ALLOW_THREADS((PyObjCObject*)proxy)->flags &=
+                ~PyObjCObject_kSHOULD_NOT_RELEASE;
     }
     Py_DECREF(proxy);
 }
