@@ -26,10 +26,8 @@ extern PyObject* _Nullable PyObjC_MakeCVoidP(void* _Nullable ptr);
 extern int PyObjCUtil_Init(PyObject* module);
 
 extern void PyObjCErr_FromObjC(NSObject* localException);
-extern void PyObjCErr_ToObjC(void) __attribute__((__noreturn__))
-__attribute__((__deprecated__("Use PyObjCErr_ToObjCWithGILState")));
 
-extern void PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nullable state)
+extern void PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nonnull state)
     __attribute__((__noreturn__));
 
 extern NSException* _Nullable PyObjCErr_AsExc(void);
@@ -39,9 +37,6 @@ extern char* _Nullable PyObjCUtil_Strdup(const char* value);
 extern NSMapTableKeyCallBacks   PyObjCUtil_PointerKeyCallBacks;
 extern NSMapTableValueCallBacks PyObjCUtil_PointerValueCallBacks;
 
-extern NSMapTableKeyCallBacks   PyObjCUtil_ObjCIdentityKeyCallBacks;
-extern NSMapTableValueCallBacks PyObjCUtil_ObjCValueCallBacks;
-
 extern void PyObjC_FreeCArray(int, Py_buffer*);
 extern int  PyObjC_PythonToCArray(BOOL, BOOL, const char*, PyObject*,
                                   void* _Nullable* _Nonnull, Py_ssize_t* _Nullable,
@@ -73,6 +68,8 @@ extern int PyObjCRT_SignaturesEqual(const char*, const char*) __attribute__((__p
 
 extern char* _Nullable PyObjC_SELToPythonName(SEL, char*, size_t);
 
+extern bool version_is_deprecated(int version);
+
 static inline Py_ssize_t
 align(Py_ssize_t offset, Py_ssize_t alignment)
 {
