@@ -110,17 +110,16 @@ PyObjCErr_FromObjC(NSObject* localException)
     PyObject*     dict;
     PyObject*     exception;
     PyObject*     v;
-    PyObject*     buf;
     PyObject*     exc_type;
     PyObject*     exc_value;
     PyObject*     exc_traceback;
     PyObject*     c_localException_name;
     PyObject*     c_localException_reason;
-    NSObject*     t;
 
+    /* XXX: add protection for ObjC exceptions */
     PyObjC_BEGIN_WITH_GIL
         if (![localException isKindOfClass:[NSException class]]) {
-            /* We caught some random objects as the exception, so the minimal possible
+            /* We caught some random objects as the exception, do the minimal possible
              */
             PyErr_SetString(PyObjCExc_Error, "non-NSException object caught");
 
@@ -166,55 +165,53 @@ PyObjCErr_FromObjC(NSObject* localException)
                 }
             }
 
-            t                     = [(NSException*)localException name];
-            c_localException_name = pythonify_c_value(@encode(NSObject*), &t);
-            if (c_localException_name == NULL) {
-                PyObjC_GIL_RETURNVOID;
+            c_localException_name = id_to_python([(NSException*)localException name]);
+            if (c_localException_name == NULL) { // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_RETURNVOID;           // LCOV_EXCL_LINE
             }
 
-            t                       = [(NSException*)localException reason];
-            c_localException_reason = pythonify_c_value(@encode(NSObject*), &t);
-            if (c_localException_reason == NULL) {
-                Py_DECREF(c_localException_name);
-                PyObjC_GIL_RETURNVOID;
+            c_localException_reason = id_to_python([(NSException*)localException reason]);
+            if (c_localException_reason == NULL) { // LCOV_BR_EXCL_LINE
+                Py_DECREF(c_localException_name);  // LCOV_EXCL_LINE
+                PyObjC_GIL_RETURNVOID;             // LCOV_EXCL_LINE
             }
 
             dict = PyDict_New();
-            if (dict == NULL) {
-                Py_DECREF(c_localException_name);
-                Py_DECREF(c_localException_reason);
-                PyObjC_GIL_RETURNVOID;
+            if (dict == NULL) {                     // LCOV_BR_EXCL_LINE
+                Py_DECREF(c_localException_name);   // LCOV_EXCL_LINE
+                Py_DECREF(c_localException_reason); // LCOV_EXCL_LINE
+                PyObjC_GIL_RETURNVOID;              // LCOV_EXCL_LINE
             }
+
+            /* Ignore errors in setting up ``dict``, the exception state
+             * will be replaced later.
+             */
             PyDict_SetItemString(dict, "name", c_localException_name);
             Py_DECREF(c_localException_name);
 
             PyDict_SetItemString(dict, "reason", c_localException_reason);
             Py_DECREF(c_localException_reason);
             if (userInfo) {
-                v = PyObjCObject_New(userInfo, PyObjCObject_kDEFAULT, YES);
+                v = id_to_python(userInfo);
                 if (v != NULL) {
                     PyDict_SetItemString(dict, "userInfo", v);
                     Py_DECREF(v);
-                } else {
-                    PyErr_Clear();
+                } else {           // LCOV_BR_EXCL_LINE
+                    PyErr_Clear(); // LCOV_EXCL_LINE
                 }
             } else {
                 PyDict_SetItemString(dict, "userInfo", Py_None);
             }
 
-            if ([[(NSException*)localException reason] UTF8String]) {
-                buf = PyUnicode_FromFormat(
-                    "%s - %s", [[(NSException*)localException name] UTF8String],
-                    [[(NSException*)localException reason] UTF8String]);
+            const char* name   = [[(NSException*)localException name] UTF8String];
+            const char* reason = [[(NSException*)localException reason] UTF8String];
+            if (reason != NULL) {
+                PyErr_Format(exception, "%s - %s", name ? name : "<null>", reason);
             } else {
-                buf = PyUnicode_FromFormat(
-                    "%s", [[(NSException*)localException name] UTF8String]);
+                PyErr_Format(exception, name ? name : "<null>");
             }
-            PyErr_SetObject(exception, buf);
             PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
-            if (!exc_value || !PyObject_IsInstance(exc_value, exc_type)) {
-                PyErr_NormalizeException(&exc_type, &exc_value, &exc_traceback);
-            }
+            PyErr_NormalizeException(&exc_type, &exc_value, &exc_traceback);
 
             PyObject_SetAttrString(exc_value, "_pyobjc_info_", dict);
             Py_CLEAR(dict);
@@ -224,13 +221,6 @@ PyObjCErr_FromObjC(NSObject* localException)
     PyObjC_END_WITH_GIL
 }
 
-/* XXX: To be removed */
-void
-PyObjCErr_ToObjC(void)
-{
-    PyObjCErr_ToObjCWithGILState(NULL);
-}
-
 NSException*
 PyObjCErr_AsExc(void)
 {
@@ -243,8 +233,12 @@ PyObjCErr_AsExc(void)
     NSException*         val;
     NSMutableDictionary* userInfo;
 
+    /* XXX: Add precondition that there is an exception,
+     *      check all callers, and enforce in debug builds
+     */
     PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
     if (exc_type == NULL) {
+        printf("No exception!\n");
         return nil;
     }
 
@@ -256,7 +250,7 @@ PyObjCErr_AsExc(void)
     } else {
         id result;
 
-        if (depythonify_c_value(@encode(id), args, &result) == -1) {
+        if (depythonify_python_object(args, &result) == -1) {
             PyErr_Clear();
             result = [[NSException alloc] initWithName:NSInternalInconsistencyException
                                                 reason:@"Cannot convert Python exception"
@@ -280,14 +274,14 @@ PyObjCErr_AsExc(void)
         /* NOTE: Don't use *WithError here because this function ignores errors */
         v = PyDict_GetItemString(args, "reason");
         if (v) {
-            if (depythonify_c_value(@encode(NSObject*), v, &reason) < 0) {
+            if (depythonify_python_object(v, &reason) < 0) {
                 PyErr_Clear();
             }
         }
 
         v = PyDict_GetItemString(args, "name");
         if (v) {
-            if (depythonify_c_value(@encode(NSObject*), v, &name) < 0) {
+            if (depythonify_python_object(v, &name) < 0) {
                 PyErr_Clear();
             }
         }
@@ -367,20 +361,20 @@ PyObjCErr_AsExc(void)
 }
 
 void
-PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nullable state)
+PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nonnull state)
 {
     NSException* exc = PyObjCErr_AsExc();
-    assert(exc != nil);
+    if (exc == nil)
+        PyObjCErr_InternalError(); // LCOV_BR_EXCL_LINE
 
     if (state) {
         PyGILState_Release(*state);
     }
-
-    [exc raise];
-
-    /* Fatal error: this should never happen */
-    NSLog(@"reached unreachable code");
-    abort();
+    if (PyGILState_Check()) {
+        printf("Still own GIL!\n");
+    }
+    @throw exc;
+    __builtin_unreachable();
 }
 
 char* _Nullable PyObjCUtil_Strdup(const char* value)
@@ -408,27 +402,6 @@ NSMapTableValueCallBacks PyObjCUtil_PointerValueCallBacks = {
     NULL,
 };
 
-static void
-nsmaptable_objc_retain(NSMapTable* table __attribute__((__unused__)), const void* datum)
-{
-    CFRetain((id)datum);
-}
-
-static void
-nsmaptable_objc_release(NSMapTable* table __attribute__((__unused__)), void* datum)
-{
-    CFRelease((id)datum);
-}
-
-NSMapTableKeyCallBacks PyObjCUtil_ObjCIdentityKeyCallBacks = {
-    NULL, NULL, &nsmaptable_objc_retain, &nsmaptable_objc_release, NULL, NULL,
-};
-
-NSMapTableValueCallBacks PyObjCUtil_ObjCValueCallBacks = {
-    &nsmaptable_objc_retain, &nsmaptable_objc_release,
-    NULL // generic description
-};
-
 #define SHOULD_FREE 1
 #define SHOULD_IGNORE 2
 
@@ -1028,10 +1001,17 @@ PyObjC_PythonToCArray(BOOL writable, BOOL exactSize, const char* elementType,
 
         if ((exactSize && seqlen != pycount) || (!exactSize && seqlen < pycount)) {
             Py_DECREF(seq);
-            PyErr_Format(PyExc_ValueError,
-                         "too few values (%" PY_FORMAT_SIZE_T "d) expecting at "
-                         "least %" PY_FORMAT_SIZE_T "d",
-                         seqlen, pycount);
+            if (exactSize) {
+                PyErr_Format(PyExc_ValueError,
+                             "expecting %" PY_FORMAT_SIZE_T "d values "
+                             "got %" PY_FORMAT_SIZE_T "d",
+                             pycount, seqlen);
+            } else {
+                PyErr_Format(PyExc_ValueError,
+                             "too few values (%" PY_FORMAT_SIZE_T "d) expecting at "
+                             "least %" PY_FORMAT_SIZE_T "d",
+                             seqlen, pycount);
+            }
             return -1;
         }
 
@@ -1517,4 +1497,10 @@ PyObject* _Nullable PyObjC_CallDecoder(PyObject* cdr, PyObject* setValue)
                                2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
 }
 
+bool
+version_is_deprecated(int version)
+{
+    return (PyObjC_DeprecationVersion && version && version <= PyObjC_DeprecationVersion);
+}
+
 NS_ASSUME_NONNULL_END
