@@ -5,7 +5,7 @@
  * Central include file for PyObjC.
  */
 
-#define OBJC_VERSION "8.1"
+#define OBJC_VERSION "8.2"
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
@@ -114,6 +114,7 @@
 #import "selector.h"
 
 #import "block-as-imp.h"
+#import "libffi_extra.h"
 #import "libffi_support.h"
 
 #import "ObjCPointer.h"
@@ -121,13 +122,11 @@
 #import "bundle-variables.h"
 #import "class-builder.h"
 #import "class-list.h"
-#import "closure_pool.h"
 #import "corefoundation.h"
 #import "ctests.h"
 #import "file_wrapper.h"
 #import "formal-protocol.h"
 #import "fsref.h"
-#import "fsspec.h"
 #import "function.h"
 #import "helpers.h"
 #import "informal-protocol.h"
