@@ -20,6 +20,7 @@
 + (CFDateRef)today;
 + (NSString*)formatDate:(CFDateRef)date;
 + (int)shortStyle;
++ (int)noStyle;
 @end
 
 @implementation OC_TestCoreFoundation
@@ -88,9 +89,9 @@
 
 + (NSString*)formatDate:(CFDateRef)date
 {
-    CFLocaleRef currentLocale = CFLocaleCopyCurrent();
-    CFDateFormatterRef formatter = CFDateFormatterCreate(
-        NULL, currentLocale, kCFDateFormatterShortStyle, kCFDateFormatterNoStyle);
+    CFLocaleRef        currentLocale = CFLocaleCopyCurrent();
+    CFDateFormatterRef formatter     = CFDateFormatterCreate(
+            NULL, currentLocale, kCFDateFormatterShortStyle, kCFDateFormatterNoStyle);
 
     if (currentLocale != NULL) {
         CFRelease(currentLocale);
@@ -108,6 +109,11 @@
     return kCFDateFormatterShortStyle;
 }
 
++ (int)noStyle
+{
+    return kCFDateFormatterNoStyle;
+}
+
 @end
 
 static PyMethodDef mod_methods[] = {{0, 0, 0, 0}};
@@ -138,7 +144,8 @@ PyObject* __attribute__((__visibility__("default"))) PyInit_corefoundation(void)
     }
 
     if (PyModule_AddObject(m, "OC_TestCoreFoundation",
-                           PyObjC_IdToPython([OC_TestCoreFoundation class])) < 0) {
+                           PyObjC_IdToPython([OC_TestCoreFoundation class]))
+        < 0) {
         return NULL;
     }
 
