@@ -6,6 +6,7 @@ Testcases for NSArchive-ing python objects.
 import pickle
 import sys
 import test.pickletester
+import collections
 
 import objc
 import objc._pycoder as pycoder
@@ -25,7 +26,7 @@ from PyObjCTest.fnd import (
 )
 from PyObjCTools.TestSupport import (
     TestCase,
-    onlyIf,
+    skipUnless,
     expectedFailure,
     expectedFailureIf,
     os_level_key,
@@ -179,12 +180,80 @@ class a_reducing_class:
         return make_instance, (self.__dict__,)
 
 
+class TestInterning(TestCase):
+    def test_intern_string(self):
+        v = "hello"
+        w = "hel"
+        w += "lo"
+        self.assertIsNot(v, w)
+        self.assertEqual(v, w)
+
+        iv = pycoder.intern(v)
+        iw = pycoder.intern(w)
+        self.assertIs(iv, iw)
+        self.assertEqual(iv, v)
+
+    def test_intern_oc_string(self):
+        v = NSString.stringWithString_("hello")
+        w = "hel"
+        w += "lo"
+
+        iv = pycoder.intern(v)
+        iw = pycoder.intern(w)
+        self.assertEqual(iv, v)
+        self.assertIsInstance(iv, str)
+        self.assertIs(iv, iw)
+
+    def test_intern_other(self):
+        v = 400000 - 1
+        w = 400000 - 2
+        v += 1
+        w += 2
+
+        self.assertIsNot(v, w)
+        self.assertEqual(v, w)
+
+        iv = pycoder.intern(v)
+        iw = pycoder.intern(w)
+
+        self.assertEqual(iv, iw)
+        self.assertEqual(iv, v)
+
+        self.assertIs(iv, v)
+        self.assertIs(iw, w)
+        self.assertIsNot(iw, iv)
+
+
 class TestKeyedArchiveSimple(TestCase):
     def setUp(self):
         self.isKeyed = True
         self.archiverClass = NSKeyedArchiver
         self.unarchiverClass = NSKeyedUnarchiver
 
+    def test_global_from_main(self):
+        import __main__ as mod
+
+        class Foo:
+            pass
+
+        Foo.__module__ = None
+        Foo.__qualname__ = "Foo"
+        mod.Foo = Foo
+        objc.Foo = "hello"
+        self.assertIs(mod.Foo.__module__, None)
+        self.assertEqual(mod.Foo.__qualname__, "Foo")
+
+        buf = self.archiverClass.archivedDataWithRootObject_(Foo)
+        self.assertIsInstance(buf, NSData)
+        v = self.unarchiverClass.unarchiveObjectWithData_(buf)
+        self.assertIs(v, Foo)
+
+        del mod.Foo
+        del objc.Foo
+
+        with self.assertRaises(pickle.PicklingError):
+            buf = self.archiverClass.archivedDataWithRootObject_(Foo)
+
     def test_unknown_type(self):
         try:
             orig = pycoder.decode_dispatch[pycoder.kOP_GLOBAL]
@@ -201,9 +270,8 @@ class TestKeyedArchiveSimple(TestCase):
             else:
                 exception = pickle.UnpicklingError
 
-            self.assertRaises(
-                exception, self.unarchiverClass.unarchiveObjectWithData_, buf
-            )
+            with self.assertRaises(exception):
+                self.unarchiverClass.unarchiveObjectWithData_(buf)
 
         finally:
             pycoder.decode_dispatch[pycoder.kOP_GLOBAL] = orig
@@ -217,7 +285,9 @@ class TestKeyedArchiveSimple(TestCase):
 
         data = NSMutableData.alloc().init()
         archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-        self.assertRaises(pickle.PicklingError, archiver.encodeRootObject_, object1)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_(object1)
+
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
@@ -229,7 +299,8 @@ class TestKeyedArchiveSimple(TestCase):
 
         data = NSMutableData.alloc().init()
         archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-        self.assertRaises(pickle.PicklingError, archiver.encodeRootObject_, object2)
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_(object2)
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
@@ -256,7 +327,8 @@ class TestKeyedArchiveSimple(TestCase):
 
             data = NSMutableData.alloc().init()
             archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-            self.assertRaises(pickle.PicklingError, archiver.encodeRootObject_, o)
+            with self.assertRaises(pickle.PicklingError):
+                archiver.encodeRootObject_(o)
 
             if self.archiverClass is NSKeyedArchiver:
                 archiver.finishEncoding()
@@ -270,7 +342,8 @@ class TestKeyedArchiveSimple(TestCase):
             o = orig("hello")
             data = NSMutableData.alloc().init()
             archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-            self.assertRaises(pickle.PicklingError, archiver.encodeRootObject_, o)
+            with self.assertRaises(pickle.PicklingError):
+                archiver.encodeRootObject_(o)
             if self.archiverClass is NSKeyedArchiver:
                 archiver.finishEncoding()
 
@@ -307,9 +380,9 @@ class TestKeyedArchiveSimple(TestCase):
                 exception = (objc.error, ValueError)
             else:
                 exception = ValueError
-            self.assertRaises(
-                exception, self.unarchiverClass.unarchiveObjectWithData_, buf
-            )
+
+            with self.assertRaises(exception):
+                self.unarchiverClass.unarchiveObjectWithData_(buf)
 
         finally:
             try:
@@ -377,9 +450,8 @@ class TestKeyedArchiveSimple(TestCase):
 
         data = NSMutableData.alloc().init()
         archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-        self.assertRaises(
-            pickle.PicklingError, archiver.encodeRootObject_, invalid_reduce()
-        )
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_(invalid_reduce())
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
@@ -389,9 +461,8 @@ class TestKeyedArchiveSimple(TestCase):
 
         data = NSMutableData.alloc().init()
         archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-        self.assertRaises(
-            pickle.PicklingError, archiver.encodeRootObject_, invalid_reduce()
-        )
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_(invalid_reduce())
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
@@ -401,9 +472,9 @@ class TestKeyedArchiveSimple(TestCase):
 
         data = NSMutableData.alloc().init()
         archiver = self.archiverClass.alloc().initForWritingWithMutableData_(data)
-        self.assertRaises(
-            pickle.PicklingError, archiver.encodeRootObject_, invalid_reduce()
-        )
+        with self.assertRaises(pickle.PicklingError):
+            archiver.encodeRootObject_(invalid_reduce())
+
         if self.archiverClass is NSKeyedArchiver:
             archiver.finishEncoding()
 
@@ -807,236 +878,236 @@ class TestKeyedArchivePlainPython(TestCase, test.pickletester.AbstractPickleTest
     # Disable a number of methods, these test things we're not interested in.
     # (Most of these look at the generated byte-stream, as we're not writing data in pickle's
     # format such tests are irrelevant to archiving support)
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_inband_accept_default_buffers_argument(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_buffers_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_bytearray(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_oob_buffers(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_in_band_buffers(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_buffer_callback_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_buffer_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_oob_buffers_writable_to_readonly(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_picklebuffer_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_negative_put(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_framed_write_sizes_with_delayed_writer(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_badly_quoted_string(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_correctly_quoted_string(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_badly_escaped_string(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_appends_on_non_lists(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_setitems_on_non_dicts(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_int_pickling_efficiency(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_dynamic_class(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_ellipsis(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_notimplemented(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_classic_instance(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_insecure_strings(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_from_canned_string(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_maxint64(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_dict_chunking(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_float_format(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_garyp(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_list_chunking(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_singletons(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_simple_newobj(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_short_tuples(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_proto(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_long1(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_long4(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_get(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_from_data0(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_from_data1(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_from_data2(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_unpickle_from_2x(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_pickle_to_2x(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_bad_getattr(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_unicode(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     # XXX: fixme: currently gives abort in debug builds of 3.4, that shouldn't happen!
     def test_unicode_high_plane(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_maxsize64(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_empty_bytestring(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_pop_empty_stack(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_framing_many_objects(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_set_chunking(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_optional_frames(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_framing_large_objects(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_long_python2_str_as_bytes(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_python2_unicode_as_str(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_load_python2_str_as_bytes(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_complex_newobj(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_compat_pickle(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_compat_unpickle(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_frame_readline(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_complex_newobj_ex(self):
         pass
 
@@ -1300,71 +1371,71 @@ class TestArchivePlainPython(TestKeyedArchivePlainPython):
         # See 'TestArchiveNative'
         test.pickletester.AbstractPickleTests.test_recursive_tuple(self)
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_oob_buffers(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_in_band_buffers(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_buffer_callback_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_buffer_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_oob_buffers_writable_to_readonly(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_picklebuffer_error(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_negative_put(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_int_pickling_efficiency(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_negative_32b_binunicode(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_negative_32b_binput(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_negative_32b_binbytes(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_framing_many_objects(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_complex_newobj(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_compat_pickle(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_compat_unpickle(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_frame_readline(self):
         pass
 
-    @onlyIf(0, "python unittest not relevant for archiving")
+    @skipUnless(0, "python unittest not relevant for archiving")
     def test_framed_write_sizes_with_delayed_writer(self):
         pass
 
@@ -1639,5 +1710,66 @@ class TestKeyedArchiveNative(TestArchiveNative):
 # And finally some tests to check if archiving of Python
 # subclasses of NSObject works correctly.
 #
+# XXX: This class is empty!
+#
 class TestArchivePythonObjCSubclass(TestCase):
     pass
+
+
+class TestSecureArchivingPython(TestCase):
+    def test_secure_archive_sequence(self):
+        archive = NSKeyedArchiver.alloc().initRequiringSecureCoding_(True)
+        sequence = collections.UserList()
+        sequence.append(1)
+        sequence.append(2)
+
+        with self.assertRaisesRegex(
+            objc.error,
+            "Class 'OC_PythonArray' disallows secure coding. It must return YES from supportsSecureCoding",
+        ):
+            archive.encodeObject_forKey_(sequence, "sequence")
+
+        archive.finishEncoding()
+
+    def test_secure_archive_mapping(self):
+        archive = NSKeyedArchiver.alloc().initRequiringSecureCoding_(True)
+        mapping = collections.UserDict()
+        mapping["key"] = 1
+
+        with self.assertRaisesRegex(
+            objc.error,
+            "Class 'OC_PythonDictionary' disallows secure coding. It must return YES from supportsSecureCoding",
+        ):
+            archive.encodeObject_forKey_(mapping, "mapping")
+
+        archive.finishEncoding()
+
+    def test_secure_archive_set(self):
+        archive = NSKeyedArchiver.alloc().initRequiringSecureCoding_(True)
+
+        class UserSet(set):
+            pass
+
+        bag = UserSet()
+        bag.add(1)
+
+        with self.assertRaisesRegex(
+            objc.error,
+            "Class 'OC_PythonSet' disallows secure coding. It must return YES from supportsSecureCoding",
+        ):
+            archive.encodeObject_forKey_(bag, "bag")
+
+        archive.finishEncoding()
+
+    def test_secure_archive_object(self):
+        archive = NSKeyedArchiver.alloc().initRequiringSecureCoding_(True)
+
+        value = object()
+
+        with self.assertRaisesRegex(
+            objc.error,
+            "This decoder will only decode classes that adopt NSSecureCoding. Class 'OC_PythonObject' does not adopt it.",
+        ):
+            archive.encodeObject_forKey_(value, "value")
+
+        archive.finishEncoding()
