@@ -1,15 +1,11 @@
-from PyObjCTools.TestSupport import *
-from PyObjCTest.classes import OCTestClasses
 import array
-import sys
+from collections import UserDict, UserList
 import datetime
+import sys
 
-if sys.version_info[0] == 3:
-    from collections import UserDict, UserList
-
-else:
-    from UserDict import UserDict
-    from UserList import UserList
+from PyObjCTest.classes import OCTestClasses
+from PyObjCTools.TestSupport import TestCase
+import objc
 
 
 class TestBridgedClasses(TestCase):
@@ -82,20 +78,18 @@ class TestBridgedClasses(TestCase):
         self.assertEqual(cls.__name__, "OC_BuiltinPythonUnicode")
 
     def test_unicode_subclass(self):
-        class U(type("")):
+        class U(str):
             pass
 
         value = U()
         cls = OCTestClasses.classForObject_(value)
         self.assertEqual(cls.__name__, "OC_PythonUnicode")
 
-    @onlyPython3
     def test_bytes(self):
         value = b""
         cls = OCTestClasses.classForObject_(value)
         self.assertEqual(cls.__name__, "OC_BuiltinPythonData")
 
-    @onlyPython3
     def test_bytes_subclass(self):
         class B(bytes):
             pass
@@ -123,7 +117,7 @@ class TestBridgedClasses(TestCase):
         self.assertEqual(cls.__name__, "OC_BuiltinPythonNumber")
 
     def test_integer_subclass(self):
-        class I(int):
+        class I(int):  # noqa: E742
             pass
 
         value = I()
@@ -172,12 +166,6 @@ class TestBridgedClasses(TestCase):
         cls = OCTestClasses.classForObject_(value)
         self.assertEqual(cls.__name__, "OC_PythonData")
 
-    @onlyPython2
-    def test_buffer(self):
-        value = buffer(b"hello")
-        cls = OCTestClasses.classForObject_(value)
-        self.assertEqual(cls.__name__, "OC_PythonData")
-
     def test_arbitrary(self):
         value = object()
         cls = OCTestClasses.classForObject_(value)
@@ -187,13 +175,9 @@ class TestBridgedClasses(TestCase):
         cls = OCTestClasses.classForObject_(value)
         self.assertEqual(cls.__name__, "OC_PythonObject")
 
-        class O(object):
+        class O:  # noqa: E742
             pass
 
         value = O()
         cls = OCTestClasses.classForObject_(value)
         self.assertEqual(cls.__name__, "OC_PythonObject")
-
-
-if __name__ == "__main__":
-    main()
