@@ -1,7 +1,7 @@
-from __future__ import absolute_import, unicode_literals
 import objc
+from PyObjCTools.TestSupport import TestCase
+
 from . import fnd as Foundation
-from PyObjCTools.TestSupport import *
 
 
 class TestBundleVariables(TestCase):
@@ -13,7 +13,10 @@ class TestBundleVariables(TestCase):
         objc.loadBundleVariables(
             self.bundle,
             d,
-            [("NSAppleScriptErrorMessage", b"@"), ("NSBundleDidLoadNotification", b"@")],
+            [
+                ("NSAppleScriptErrorMessage", b"@"),
+                ("NSBundleDidLoadNotification", b"@"),
+            ],
         )
 
         self.assertIn("NSBundleDidLoadNotification", d)
@@ -38,7 +41,3 @@ class TestBundleVariables(TestCase):
 
         self.assertIsInstance(d["NSFoundationVersionNumber"], float)
         self.assertIsInstance(d["NSDebugEnabled"], int)
-
-
-if __name__ == "__main__":
-    main()
