@@ -1,12 +1,14 @@
-import operator
 import os
 import struct
 import sys
+import copy
+import warnings
 
 import objc
 import objc._lazyimport as lazyimport
 from PyObjCTest import metadatafunction
 from PyObjCTools.TestSupport import TestCase
+from PyObjCTest.test_deprecations import deprecation_warnings
 
 
 def lookupClasses(*names):
@@ -28,7 +30,11 @@ class TestLazyImport(TestCase):
         o = lazyimport.GetAttrMap(sys)
         self.assertEqual(o["path"], sys.path)
         self.assertEqual(o["version"], sys.version)
-        self.assertRaises(KeyError, o.__getitem__, "nosuchkey")
+        with self.assertRaisesRegex(KeyError, "nosuchkey"):
+            o["nosuchkey"]
+
+        v = o["CFSTR"]
+        self.assertEqual(v(b"hello"), "hello")
 
     def test_load_bundle(self):
         NSBundle = objc.lookUpClass("NSBundle")
@@ -122,9 +128,12 @@ class TestLazyImport(TestCase):
         )
         self.assertIsInstance(mod, objc.ObjCLazyModule)
 
-        self.assertRaises(AttributeError, getattr, mod, "Foo(")
-        self.assertRaises(AttributeError, getattr, mod, "Foo)")
-        self.assertRaises(AttributeError, getattr, mod, "42")
+        with self.assertRaisesRegex(AttributeError, "Foo"):
+            mod.Foo
+        with self.assertRaisesRegex(AttributeError, "Foo"):
+            mod.Foo
+        with self.assertRaisesRegex(AttributeError, "42"):
+            getattr(mod, "42")
 
         if dunder_all:
             # Force precalculation of all attributes by accessing the __all__
@@ -133,7 +142,8 @@ class TestLazyImport(TestCase):
 
         self.assertEqual(mod.__doc__, initial_dict["__doc__"])
         self.assertEqual(mod.doc_string, initial_dict["__doc__"])
-        self.assertRaises(AttributeError, getattr, mod, "invalid_alias")
+        with self.assertRaisesRegex(AttributeError, "invalid_alias"):
+            mod.invalid_alias
         self.assertIsInstance(mod.NSWorkspaceMoveOperation, objc.pyobjc_unicode)
         self.assertTrue(
             (mod.NSWorkspaceMoveOperation.nsstring().__flags__ & 0x10) == 0x00
@@ -148,12 +158,16 @@ class TestLazyImport(TestCase):
         self.assertIsInstance(mod.NSRectClipList, objc.function)
         self.assertEqual(mod.NSRectClipList.__name__, "NSRectClipList")
         self.assertArgSizeInArg(mod.NSRectClipList, 0, 1)
-        self.assertRaises(AttributeError, getattr, mod, "FunctionThatDoesNotExist")
+        with self.assertRaisesRegex(AttributeError, "FunctionThatDoesNotExist"):
+            mod.FunctionThatDoesNotExist
         self.assertEqual(mod.mysum, mod.NSAWTEventType + mod.NSAboveBottom + 3)
-        self.assertRaises(AttributeError, getattr, mod, "invalid_expression1")
-        self.assertRaises(AttributeError, getattr, mod, "invalid_expression2")
+        with self.assertRaisesRegex(AttributeError, "invalid_expression1"):
+            mod.invalid_expression1
+        with self.assertRaisesRegex(AttributeError, "invalid_expression2"):
+            mod.invalid_expression2
         self.assertIs(mod.NSURL, objc.lookUpClass("NSURL"))
-        self.assertRaises(AttributeError, getattr, mod, "NSNonExistingClass")
+        with self.assertRaisesRegex(AttributeError, "NSNonExistingClass"):
+            mod.NSNonExistingClass
 
         mod.NSAccessibilityActionDescription = 99
         mod.NSWindowWillCloseNotification = 100
@@ -199,11 +213,12 @@ class TestLazyImport(TestCase):
         self.assertIs(mod.__loader__, initial_dict["__loader__"])
         self.assertEqual(mod.NSAboveBottom, 4)
         self.assertEqual(mod.mysum, mod.NSAWTEventType + mod.NSAboveBottom + 3)
-        self.assertRaises(AttributeError, getattr, mod, "ABPersonSetImageData")
-        self.assertRaises(AttributeError, getattr, mod, "ABAddressBookErrorDomain")
-        self.assertRaises(
-            AttributeError, getattr, mod, "ABMultiValueIdentifiersErrorKey"
-        )
+        with self.assertRaisesRegex(AttributeError, "ABPersonSetImageData"):
+            mod.ABPersonSetImageData
+        with self.assertRaisesRegex(AttributeError, "ABAddressBookErrorDomain"):
+            mod.ABAddressBookErrorDomain
+        with self.assertRaisesRegex(AttributeError, "ABMultiValueIdentifiersErrorKey"):
+            mod.ABMultiValueIdentifiersErrorKey
 
     def test_with_parents(self):
         mod = objc.ObjCLazyModule("RootLess", None, None, None, None, None, (sys, os))
@@ -274,7 +289,17 @@ class TestLazyImport(TestCase):
 
     def test_magic_aliases(self):
         metadict = {
-            "aliases": {"umax": "ULONG_MAX", "max": "LONG_MAX", "min": "LONG_MIN"}
+            "aliases": {
+                "umax": "ULONG_MAX",
+                "max": "LONG_MAX",
+                "min": "LONG_MIN",
+                "dblmx": "DBL_MAX",
+                "dblmn": "DBL_MIN",
+                "fltmx": "FLT_MAX",
+                "fltmn": "FLT_MIN",
+                "null": "objc.NULL",
+                "umx": "UINT32_MAX",
+            }
         }
 
         initial_dict = {"__doc__": "AppKit test module"}
@@ -293,6 +318,12 @@ class TestLazyImport(TestCase):
         self.assertEqual(mod.umax, 2 ** 64 - 1)
         self.assertEqual(mod.max, sys.maxsize)
         self.assertEqual(mod.min, -sys.maxsize - 1)
+        self.assertEqual(mod.dblmx, sys.float_info.max)
+        self.assertEqual(mod.dblmn, sys.float_info.min)
+        self.assertEqual(mod.fltmx, objc._FLT_MAX)
+        self.assertEqual(mod.fltmn, objc._FLT_MIN)
+        self.assertEqual(mod.null, objc.NULL)
+        self.assertEqual(mod.umx, 2 ** 32 - 1)
 
     def test_existing_submodules(self):
         try:
@@ -304,8 +335,10 @@ class TestLazyImport(TestCase):
             self.assertIsInstance(mod, objc.ObjCLazyModule)
             self.assertEqual(mod.submodule, 42)
             self.assertEqual(mod.submodule2, 1)
-            self.assertRaises(KeyError, operator.getitem, mod.__dict__, "submodule3")
-            self.assertRaises(KeyError, operator.getitem, mod.__dict__, "submodule.x")
+            with self.assertRaisesRegex(KeyError, "submodule3"):
+                mod.__dict__["submodule3"]
+            with self.assertRaisesRegex(KeyError, "x"):
+                mod.__dict__["submodule.x"]
         finally:
             for nm in (
                 "MyFramework.submodule",
@@ -356,13 +389,61 @@ class TestLazyImport(TestCase):
         self.assertIsInstance(mod.makeArrayWithFormat_, objc.function)
         v = mod.makeArrayWithFormat_("%3d", 10)
         self.assertEqual(list(v), ["%3d", " 10"])
-        self.assertRaises(AttributeError, getattr, mod, "NoSuchFunction")
+        with self.assertRaisesRegex(AttributeError, "NoSuchFunction"):
+            mod.NoSuchFunction
 
         mod.make4Tuple_ = 42
         self.assertIn("makeArrayWithFormat_", mod.__all__)
         self.assertIn("makeArrayWithCFormat_", mod.__all__)
         self.assertEqual(mod.make4Tuple_, 42)
 
+    def test_inline_list__all__(self):
+        # Check __all__ handling for inline functions
+        metadict = {
+            "functions": {
+                "makeArrayWithFormat_": (
+                    b"@@",
+                    "",
+                    {"variadic": True, "arguments": {0: {"printf_format": True}}},
+                ),
+                "makeArrayWithCFormat_": (
+                    b"@*",
+                    "",
+                    {"variadic": True, "arguments": {0: {"printf_format": True}}},
+                ),
+                "make4Tuple_": (
+                    b"@^d",
+                    "",
+                    {
+                        "arguments": {
+                            0: {
+                                "type_modifier": objc._C_IN,
+                                "c_array_of_fixed_length": 4,
+                                "null_accepted": False,
+                            }
+                        }
+                    },
+                ),
+                "NoSuchFunction": (b"@d", "", {}),
+            }
+        }
+
+        inline_list = metadatafunction.function_list
+        mod = objc.ObjCLazyModule(
+            "MyFramework", None, None, metadict, inline_list, {}, ()
+        )
+        self.assertIsInstance(mod, objc.ObjCLazyModule)
+        mod.__dict__["makeArrayWithFormat_"] = 42
+        mod.__dict__["make4Tuple_"] = 43
+
+        self.assertIn("makeArrayWithFormat_", mod.__all__)
+        self.assertIn("makeArrayWithCFormat_", mod.__all__)
+        self.assertIn("make4Tuple_", mod.__all__)
+
+        self.assertEqual(mod.makeArrayWithFormat_, 42)
+        self.assertEqual(mod.make4Tuple_, 43)
+        self.assertIsInstance(mod.makeArrayWithCFormat_, objc.function)
+
     def test_cftype(self):
         metadict = {
             "cftypes": [
@@ -432,27 +513,42 @@ class TestLazyImport(TestCase):
         self.assertFalse(mod.kCFAllocatorDefault != mod.kCFAllocatorDefault)
         self.assertTrue(mod.kCFAllocatorDefault != mod.CFBagRef)
         self.assertFalse(mod.kCFAllocatorDefault == mod.CFBagRef)
-        with self.assertRaises(TypeError):
+        with self.assertRaisesRegex(
+            TypeError,
+            "'<' not supported between instances of 'CFAllocatorRef' and 'CFAllocatorRef'",
+        ):
             mod.kCFAllocatorDefault < mod.kCFAllocatorDefault  # noqa: B015
-        with self.assertRaises(TypeError):
+        with self.assertRaisesRegex(
+            TypeError,
+            "'<=' not supported between instances of 'CFAllocatorRef' and 'CFAllocatorRef'",
+        ):
             mod.kCFAllocatorDefault <= mod.kCFAllocatorDefault  # noqa: B015
-        with self.assertRaises(TypeError):
+        with self.assertRaisesRegex(
+            TypeError,
+            "'>' not supported between instances of 'CFAllocatorRef' and 'CFAllocatorRef'",
+        ):
             mod.kCFAllocatorDefault > mod.kCFAllocatorDefault  # noqa: B015
-        with self.assertRaises(TypeError):
+        with self.assertRaisesRegex(
+            TypeError,
+            "'>=' not supported between instances of 'CFAllocatorRef' and 'CFAllocatorRef'",
+        ):
             mod.kCFAllocatorDefault >= mod.kCFAllocatorDefault  # noqa: B015
 
         self.assertIsInstance(mod.kCFAllocatorSystemDefault, objc.objc_object)
         self.assertTrue((mod.kCFAllocatorSystemDefault.__flags__ & 0x10) == 0x10)
         self.assertIsInstance(mod.kCFAllocatorSystemDefault, mod.CFAllocatorRef)
 
-        self.assertRaises(AttributeError, getattr, mod, "kCFAllocatorMissing")
-        self.assertRaises(AttributeError, getattr, mod, "kCFAllocatorMissingZone")
+        with self.assertRaisesRegex(AttributeError, "kCFAllocatorMissing"):
+            mod.kCFAllocatorMissing
+        with self.assertRaisesRegex(AttributeError, "kCFAllocatorMissingZone"):
+            mod.kCFAllocatorMissingZone
 
         self.assertIn("kCFAllocatorDefault", mod.__all__)
         self.assertIn("kCFAllocatorSystemDefault", mod.__all__)
         self.assertIn("kCFAllocatorMallocZone", mod.__all__)
         self.assertIn("kCFAllocatorMalloc", mod.__all__)
-        self.assertRaises(AttributeError, getattr, mod, "kCFAllocatorOtherMissingZone")
+        with self.assertRaisesRegex(AttributeError, "kCFAllocatorOtherMissingZone"):
+            mod.kCFAllocatorOtherMissingZone
 
         self.assertIsInstance(mod.kCFAllocatorMalloc, objc.objc_object)
         self.assertTrue((mod.kCFAllocatorMalloc.__flags__ & 0x10) == 0x10)
@@ -461,3 +557,216 @@ class TestLazyImport(TestCase):
         self.assertIsInstance(mod.kCFAllocatorMallocZone, objc.objc_object)
         self.assertTrue((mod.kCFAllocatorMallocZone.__flags__ & 0x10) == 0x10)
         self.assertIsInstance(mod.kCFAllocatorMallocZone, mod.CFAllocatorRef)
+
+    def assertDeprecationWarning(self, func):
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            func()
+
+        self.assertEqual(len(w), 1)
+        self.assertTrue(issubclass(w[-1].category, objc.ApiDeprecationWarning))
+
+    def assertNoDeprecationWarning(self, func):
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter("always")
+            func()
+
+        self.assertEqual(len(w), 0)
+
+    def test_deprecatins(self):
+        metadict = {
+            "constants": "$NSWorkspaceMoveOperation$NSWorkspaceCopyOperation@@$",
+            "constants_dict": {
+                "NSWorkspaceLinkOperation": "@",
+                "NSWindowWillCloseNotification": "@",
+                "NSUnderlineByWordMask": objc._C_NSUInteger.decode(),
+            },
+            "enums": "$NSAWTEventType@16$NSAboveBottom@4$NSAboveTop@1$",
+            "aliases": {"min": "LONG_MIN", "max": "LONG_MAX"},
+            "deprecated_aliases": {
+                "min": 1004,
+                "max": 1008,
+            },
+            "deprecated_constants": {
+                "NSWorkspaceLinkOperation": 1004,
+                "NSWorkspaceMoveOperation": 1008,
+            },
+            "deprecated_enums": {
+                "NSAWTEventType": 1004,
+                "NSAboveBottom": 1008,
+            },
+        }
+
+        def make_mod():
+            initial_dict = {"__doc__": "AppKit test module"}
+
+            return objc.ObjCLazyModule(
+                "AppKit",
+                None,
+                "/System/Library/Frameworks/AppKit.framework",
+                copy.deepcopy(metadict),
+                None,
+                initial_dict,
+                (),
+            )
+
+        with deprecation_warnings(1003):
+            mod = make_mod()
+            self.assertIsInstance(mod, objc.ObjCLazyModule)
+            self.assertNoDeprecationWarning(lambda: mod.NSWorkspaceLinkOperation)
+            self.assertNoDeprecationWarning(lambda: mod.NSWorkspaceMoveOperation)
+            self.assertNoDeprecationWarning(lambda: mod.NSAWTEventType)
+            self.assertNoDeprecationWarning(lambda: mod.NSAboveBottom)
+            self.assertNoDeprecationWarning(lambda: mod.min)
+            self.assertNoDeprecationWarning(lambda: mod.max)
+
+        with deprecation_warnings(1005):
+            mod = make_mod()
+            self.assertIsInstance(mod, objc.ObjCLazyModule)
+            self.assertDeprecationWarning(lambda: mod.NSWorkspaceLinkOperation)
+            self.assertNoDeprecationWarning(lambda: mod.NSWorkspaceMoveOperation)
+            self.assertDeprecationWarning(lambda: mod.NSAWTEventType)
+            self.assertNoDeprecationWarning(lambda: mod.NSAboveBottom)
+            self.assertDeprecationWarning(lambda: mod.min)
+            self.assertNoDeprecationWarning(lambda: mod.max)
+
+        with deprecation_warnings(1200):
+            mod = make_mod()
+            self.assertIsInstance(mod, objc.ObjCLazyModule)
+            self.assertDeprecationWarning(lambda: mod.NSWorkspaceLinkOperation)
+            self.assertDeprecationWarning(lambda: mod.NSWorkspaceMoveOperation)
+            self.assertDeprecationWarning(lambda: mod.NSAWTEventType)
+            self.assertDeprecationWarning(lambda: mod.NSAboveBottom)
+            self.assertDeprecationWarning(lambda: mod.min)
+            self.assertDeprecationWarning(lambda: mod.max)
+
+    def test_functions_all(self):
+
+        for override in (False, True):
+            metadict = {
+                "functions": {
+                    "LSSharedFileListItemGetTypeID": (b"Q",),
+                },
+            }
+            initial_dict = {"__doc__": "AppKit test module"}
+
+            mod = objc.ObjCLazyModule(
+                "AppKit",
+                None,
+                "/System/Library/Frameworks/AppKit.framework",
+                copy.deepcopy(metadict),
+                None,
+                initial_dict,
+                (),
+            )
+
+            if override:
+                mod.LSSharedFileListItemGetTypeID = 42
+                mod.__all__
+                self.assertEqual(mod.LSSharedFileListItemGetTypeID, 42)
+            else:
+                self.assertIsInstance(mod.LSSharedFileListItemGetTypeID, objc.function)
+
+    def do_indirect_magic(self, fetchall):
+        metadict = {
+            "cftypes": [
+                (
+                    "LSSharedFileListItemRef",
+                    b"^{OpaqueLSSharedFileListItemRef=}",
+                    "LSSharedFileListItemGetTypeID",
+                    None,
+                ),
+            ],
+            "functions": {
+                "LSSharedFileListItemGetTypeID": (b"Q",),
+            },
+            "constants": "$kLSSharedFileListItemBeforeFirst@==^{OpaqueLSSharedFileListItemRef=}$",
+        }
+        initial_dict = {"__doc__": "AppKit test module"}
+
+        mod = objc.ObjCLazyModule(
+            "AppKit",
+            None,
+            "/System/Library/Frameworks/AppKit.framework",
+            copy.deepcopy(metadict),
+            None,
+            initial_dict,
+            (),
+        )
+        if fetchall:
+            mod.__all__
+
+        self.assertIsInstance(
+            mod.kLSSharedFileListItemBeforeFirst, mod.LSSharedFileListItemRef
+        )
+
+    def test_indirect_magic(self):
+        self.do_indirect_magic(False)
+        self.do_indirect_magic(True)
+
+    def do_indirect_magic_dict(self, fetchall):
+        metadict = {
+            "cftypes": [
+                (
+                    "LSSharedFileListItemRef",
+                    b"^{OpaqueLSSharedFileListItemRef=}",
+                    "LSSharedFileListItemGetTypeID",
+                    None,
+                ),
+            ],
+            "functions": {
+                "LSSharedFileListItemGetTypeID": (b"Q",),
+            },
+            "constants_dict": {
+                "kLSSharedFileListItemBeforeFirst": "==^{OpaqueLSSharedFileListItemRef=}",
+                "kLSSharedFileListItemLaatste": "==^{OpaqueLSSharedFileListItemRef=}",
+            },
+        }
+        initial_dict = {"__doc__": "AppKit test module"}
+
+        mod = objc.ObjCLazyModule(
+            "AppKit",
+            None,
+            "/System/Library/Frameworks/AppKit.framework",
+            copy.deepcopy(metadict),
+            None,
+            initial_dict,
+            (),
+        )
+
+        if fetchall:
+            mod.__all__
+
+        self.assertIsInstance(
+            mod.kLSSharedFileListItemBeforeFirst, mod.LSSharedFileListItemRef
+        )
+        self.assertNotHasAttr(mod, "kLSSharedFileListItemLaatste")
+
+    def test_indirect_magic_dict(self):
+        self.do_indirect_magic_dict(False)
+        self.do_indirect_magic_dict(True)
+
+    def test_default_cftype(self):
+        metadict = {
+            "cftypes": [
+                (
+                    "LSSharedFileListItemRef",
+                    b"^{OpaqueLSSharedFileListItemRef=}",
+                    None,
+                    None,
+                ),
+            ],
+        }
+        initial_dict = {"__doc__": "AppKit test module"}
+
+        mod = objc.ObjCLazyModule(
+            "AppKit",
+            None,
+            "/System/Library/Frameworks/AppKit.framework",
+            copy.deepcopy(metadict),
+            None,
+            initial_dict,
+            (),
+        )
+
+        self.assertIn("CFType", mod.LSSharedFileListItemRef.__name__)
