@@ -4,17 +4,14 @@ Tests for the proxy of Python numbers
 NOTE: Decimal conversion is not tested, the required proxy is part of
 the Foundation bindings :-(
 """
-from __future__ import unicode_literals
-import sys, os
-from PyObjCTools.TestSupport import *
-from PyObjCTest.fnd import NSNumber, NSNumberFormatter
-from PyObjCTest.pythonnumber import OC_TestNumber
+import sys
+import warnings
+
 import objc
-
-
-if sys.version_info[0] == 3:
-    unicode = str
-    long = int
+from PyObjCTest.fnd import NSNumber, NSNumberFormatter
+from PyObjCTest.misc import OC_Misc
+from PyObjCTest.pythonnumber import OC_TestNumber
+from PyObjCTools.TestSupport import TestCase, os_level_key, os_release
 
 OC_PythonNumber = objc.lookUpClass("OC_PythonNumber")
 OC_BuiltinPythonNumber = objc.lookUpClass("OC_BuiltinPythonNumber")
@@ -43,7 +40,7 @@ class TestNSNumber(TestCase):
         ):
             v = getattr(NSNumber, m)(0)
             self.assertIsInstance(v, NSNumber)
-            self.assertIsNotInstance(v, OC_PythonNumber)
+            self.assertNotIsInstance(v, OC_PythonNumber)
             self.assertIs(OC_TestNumber.numberClass_(v), NSCFNumber)
 
     def testDecimal(self):
@@ -58,16 +55,19 @@ class TestNSNumber(TestCase):
 
     def testLongValue(self):
         v = NSNumber.numberWithUnsignedLongLong_(2 ** 63 + 5000)
-        self.assertIsInstance(v, long)
+        self.assertIsInstance(v, int)
 
         if os_level_key(os_release()) < os_level_key("10.5"):
-            self.assertEqual(v.description(), str(-2 ** 63 + 5000))
+            self.assertEqual(v.description(), str(-(2 ** 63) + 5000))
         else:
             self.assertEqual(v.description(), str(2 ** 63 + 5000))
 
-        self.assertIsNot(type(v), long)
+        self.assertIsNot(type(v), int)
 
-        self.assertRaises(AttributeError, setattr, v, "x", 42)
+        with self.assertRaisesRegex(
+            AttributeError, "'.*NSCFNumber' object has no attribute 'x'"
+        ):
+            v.x = 42
 
     def testEdgeCases(self):
         from objc._pythonify import numberWrapper
@@ -111,7 +111,7 @@ class TestNSNumber(TestCase):
 
         w = pickle.loads(data)
         if os_level_key(os_release()) < os_level_key("10.5"):
-            self.assertEqual(w, {"long": -2 ** 63 + 5000, "int": 42, "float": 2.0})
+            self.assertEqual(w, {"long": -(2 ** 63) + 5000, "int": 42, "float": 2.0})
         else:
             self.assertEqual(w, {"long": 2 ** 63 + 5000, "int": 42, "float": 2.0})
 
@@ -227,8 +227,12 @@ class TestNSNumber(TestCase):
         self.assertEqual(OC_TestNumber.numberAsUnsignedShort_(v), 65409)
         self.assertEqual(OC_TestNumber.numberAsUnsignedInt_(v), 4_294_967_169)
 
-        if sys.maxsize == (2 ** 31) - 1:
-            self.assertEqual(OC_TestNumber.numberAsUnsignedLong_(v), 4_294_967_169)
+        # NOTE: The expected values in the test below were determined by running
+        #       the equivalent ObjC code.
+        if objc.arch == "arm64":
+            self.assertEqual(
+                OC_TestNumber.numberAsUnsignedLong_(v), 18_446_744_073_709_551_615
+            )
         else:
             self.assertEqual(
                 OC_TestNumber.numberAsUnsignedLong_(v), 18_446_744_073_709_551_488
@@ -245,7 +249,11 @@ class TestNSNumber(TestCase):
 
         self.assertIn(
             OC_TestNumber.numberAsUnsignedLongLong_(v),
-            (18_446_744_073_709_551_489, 18_446_744_073_709_551_488),
+            (
+                18_446_744_073_709_551_489,
+                18_446_744_073_709_551_488,
+                18_446_744_073_709_551_615,
+            ),
         )
 
         self.assertEqual(OC_TestNumber.numberAsDouble_(v), -127.6)
@@ -255,7 +263,7 @@ class TestNSNumber(TestCase):
 
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
 
-        if sys.byteorder == "big":
+        if objc.arch == "arm64":
             self.assertEqual(OC_TestNumber.numberAsChar_(v), -1)
             self.assertEqual(OC_TestNumber.numberAsShort_(v), -1)
             self.assertEqual(OC_TestNumber.numberAsUnsignedChar_(v), 255)
@@ -275,7 +283,8 @@ class TestNSNumber(TestCase):
         )
         self.assertEqual(
             OC_TestNumber.compareA_andB_(
-                NSNumber.numberWithLong_(0), NSNumber.numberWithUnsignedLongLong_(2 ** 40)
+                NSNumber.numberWithLong_(0),
+                NSNumber.numberWithUnsignedLongLong_(2 ** 40),
             ),
             NSOrderedAscending,
         )
@@ -294,7 +303,7 @@ class TestNSNumber(TestCase):
         )
         self.assertEqual(
             OC_TestNumber.compareA_andB_(
-                NSNumber.numberWithLong_(0), NSNumber.numberWithLongLong_(-2 ** 60)
+                NSNumber.numberWithLong_(0), NSNumber.numberWithLongLong_(-(2 ** 60))
             ),
             NSOrderedDescending,
         )
@@ -326,19 +335,19 @@ class TestNSNumber(TestCase):
 
     def testDescription(self):
         v = OC_TestNumber.numberDescription_(NSNumber.numberWithInt_(0))
-        self.assertIsInstance(v, unicode)
+        self.assertIsInstance(v, str)
         self.assertEqual(v, "0")
 
         v = OC_TestNumber.numberDescription_(NSNumber.numberWithLongLong_(2 ** 60))
-        self.assertIsInstance(v, unicode)
-        self.assertEqual(v, unicode(str(2 ** 60)))
+        self.assertIsInstance(v, str)
+        self.assertEqual(v, str(2 ** 60))
 
-        v = OC_TestNumber.numberDescription_(NSNumber.numberWithLongLong_(-2 ** 60))
-        self.assertIsInstance(v, unicode)
-        self.assertEqual(v, unicode(str(-2 ** 60)))
+        v = OC_TestNumber.numberDescription_(NSNumber.numberWithLongLong_(-(2 ** 60)))
+        self.assertIsInstance(v, str)
+        self.assertEqual(v, str(-(2 ** 60)))
 
         v = OC_TestNumber.numberDescription_(NSNumber.numberWithDouble_(264.0))
-        self.assertIsInstance(v, unicode)
+        self.assertIsInstance(v, str)
         self.assertEqual(v, "264")
 
 
@@ -362,6 +371,26 @@ class TestPyNumber(TestCase):
             self.assertIs(OC_TestNumber.numberClass_(v), boolClass)
             self.assertIs(objc.repythonify(v), v)
 
+    def test_repythonify_invalid(self):
+        with self.assertRaisesRegex(
+            TypeError,
+            r"(function missing required argument 'obj' \(pos 1\))|(Required argument 'obj' \(pos 1\) not found)",
+        ):
+            objc.repythonify()
+
+        with self.assertRaisesRegex(
+            TypeError, "a bytes-like object is required, not 'str'"
+        ):
+            objc.repythonify(42, "i")
+
+        with self.assertRaisesRegex(
+            objc.internal_error, "PyObjCRT_SizeOfType: Unhandled type '0x5f', _"
+        ):
+            objc.repythonify(42, b"_")
+
+        with self.assertRaisesRegex(ValueError, "depythonifying 'int', got 'str' of 1"):
+            objc.repythonify("a", b"i")
+
     def testPythonIntConversions(self):
         # Conversions to other values. Note that values are converted
         # using C casts, without any exceptions when converting a
@@ -419,8 +448,8 @@ class TestPyNumber(TestCase):
         self.assertEqual(OC_TestNumber.numberAsUnsignedChar_(v), 39)
         self.assertEqual(OC_TestNumber.numberAsUnsignedShort_(v), 40487)
 
-        # Python long
-        v = long(42)
+        # Python integer
+        v = 42
 
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
         self.assertEqual(OC_TestNumber.numberAsChar_(v), 42)
@@ -438,11 +467,7 @@ class TestPyNumber(TestCase):
         self.assertEqual(OC_TestNumber.numberAsDouble_(v), 42.0)
 
     def testPythonLongConversions(self):
-        if sys.version_info[0] == 2:
-            v = long(42)
-            self.assertIsInstance(v, long)
-        else:
-            v = 42
+        v = 42
 
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
         self.assertEqual(OC_TestNumber.numberAsChar_(v), 42)
@@ -460,11 +485,7 @@ class TestPyNumber(TestCase):
         self.assertEqual(OC_TestNumber.numberAsDouble_(v), 42.0)
 
         # Negative values
-        if sys.version_info[0] == 2:
-            v = long(-42)
-            self.assertIsInstance(v, long)
-        else:
-            v = -42
+        v = -42
 
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
         self.assertEqual(OC_TestNumber.numberAsChar_(v), -42)
@@ -490,11 +511,7 @@ class TestPyNumber(TestCase):
         self.assertEqual(OC_TestNumber.numberAsDouble_(v), -42.0)
 
         # Overflow
-        if sys.version_info[0] == 2:
-            v = long(892_455)
-            self.assertIsInstance(v, long)
-        else:
-            v = 892_455
+        v = 892_455
 
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
         self.assertEqual(OC_TestNumber.numberAsChar_(v), 39)
@@ -568,7 +585,7 @@ class TestPyNumber(TestCase):
 
         self.assertEqual(OC_TestNumber.numberAsBOOL_(v), 1)
 
-        if sys.byteorder == "big":
+        if objc.arch == "arm64":
             self.assertEqual(OC_TestNumber.numberAsChar_(v), -1)
             self.assertEqual(OC_TestNumber.numberAsShort_(v), -1)
             self.assertEqual(OC_TestNumber.numberAsUnsignedChar_(v), 255)
@@ -585,13 +602,13 @@ class TestPyNumber(TestCase):
         self.assertEqual(OC_TestNumber.compareA_andB_(0, 42.0), NSOrderedAscending)
 
         self.assertEqual(OC_TestNumber.compareA_andB_(0, -1), NSOrderedDescending)
-        self.assertEqual(OC_TestNumber.compareA_andB_(0, -2 ** 64), NSOrderedDescending)
+        self.assertEqual(
+            OC_TestNumber.compareA_andB_(0, -(2 ** 64)), NSOrderedDescending
+        )
         self.assertEqual(OC_TestNumber.compareA_andB_(0, -42.0), NSOrderedDescending)
 
         self.assertEqual(OC_TestNumber.compareA_andB_(0, 0), NSOrderedSame)
         self.assertEqual(OC_TestNumber.compareA_andB_(0, 0.0), NSOrderedSame)
-        if sys.version_info[0] == 2:
-            self.assertEqual(OC_TestNumber.compareA_andB_(0, long(0)), NSOrderedSame)
 
     def testNumberEqual(self):
         self.assertFalse(OC_TestNumber.number_isEqualTo_(0, 1))
@@ -599,37 +616,35 @@ class TestPyNumber(TestCase):
         self.assertFalse(OC_TestNumber.number_isEqualTo_(0, 42.0))
 
         self.assertFalse(OC_TestNumber.number_isEqualTo_(0, -1))
-        self.assertFalse(OC_TestNumber.number_isEqualTo_(0, -2 ** 64))
+        self.assertFalse(OC_TestNumber.number_isEqualTo_(0, -(2 ** 64)))
         self.assertFalse(OC_TestNumber.number_isEqualTo_(0, -42.0))
 
         self.assertTrue(OC_TestNumber.number_isEqualTo_(0, 0))
         self.assertTrue(OC_TestNumber.number_isEqualTo_(0, 0.0))
-        if sys.version_info[0] == 2:
-            self.assertTrue(OC_TestNumber.number_isEqualTo_(0, long(0)))
 
     def testDescription(self):
         v = OC_TestNumber.numberDescription_(0)
-        self.assertIsInstance(v, unicode)
+        self.assertIsInstance(v, str)
         self.assertEqual(v, "0")
 
         v = OC_TestNumber.numberDescription_(2 ** 64)
-        self.assertIsInstance(v, unicode)
-        self.assertEqual(v, unicode(repr(2 ** 64)))
+        self.assertIsInstance(v, str)
+        self.assertEqual(v, repr(2 ** 64))
 
-        v = OC_TestNumber.numberDescription_(-2 ** 64)
-        self.assertIsInstance(v, unicode)
-        self.assertEqual(v, unicode(repr(-2 ** 64)))
+        v = OC_TestNumber.numberDescription_(-(2 ** 64))
+        self.assertIsInstance(v, str)
+        self.assertEqual(v, repr(-(2 ** 64)))
 
         v = OC_TestNumber.numberDescription_(264.0)
-        self.assertIsInstance(v, unicode)
+        self.assertIsInstance(v, str)
         self.assertEqual(v, "264.0")
 
         v = OC_TestNumber.numberDescription_(False)
-        self.assertIsInstance(v, unicode)
+        self.assertIsInstance(v, str)
         self.assertEqual(v, "0")
 
         v = OC_TestNumber.numberDescription_(True)
-        self.assertIsInstance(v, unicode)
+        self.assertIsInstance(v, str)
         self.assertEqual(v, "1")
 
 
@@ -651,10 +666,12 @@ class TestInteractions(TestCase):
         self.assertEqual(OC_TestNumber.compareA_andB_(42, 42), NSOrderedSame)
         for m in methods:
             self.assertEqual(
-                OC_TestNumber.compareA_andB_(getattr(NSNumber, m)(42), 42), NSOrderedSame
+                OC_TestNumber.compareA_andB_(getattr(NSNumber, m)(42), 42),
+                NSOrderedSame,
             )
             self.assertEqual(
-                OC_TestNumber.compareA_andB_(42, getattr(NSNumber, m)(42)), NSOrderedSame
+                OC_TestNumber.compareA_andB_(42, getattr(NSNumber, m)(42)),
+                NSOrderedSame,
             )
 
         self.assertEqual(OC_TestNumber.compareA_andB_(42, 99), NSOrderedAscending)
@@ -675,18 +692,30 @@ class TestInteractions(TestCase):
         # For: (bool, int, long, float) vs (char, short, ...)
         self.assertTrue(OC_TestNumber.number_isEqualTo_(0, NSNumber.numberWithInt_(0)))
         self.assertTrue(OC_TestNumber.number_isEqualTo_(0, NSNumber.numberWithLong_(0)))
-        self.assertTrue(OC_TestNumber.number_isEqualTo_(0, NSNumber.numberWithFloat_(0)))
+        self.assertTrue(
+            OC_TestNumber.number_isEqualTo_(0, NSNumber.numberWithFloat_(0))
+        )
         self.assertTrue(OC_TestNumber.number_isEqualTo_(NSNumber.numberWithInt_(0), 0))
         self.assertTrue(OC_TestNumber.number_isEqualTo_(NSNumber.numberWithLong_(0), 0))
-        self.assertTrue(OC_TestNumber.number_isEqualTo_(NSNumber.numberWithFloat_(0), 0))
+        self.assertTrue(
+            OC_TestNumber.number_isEqualTo_(NSNumber.numberWithFloat_(0), 0)
+        )
 
-        self.assertFalse(OC_TestNumber.number_isEqualTo_(42, NSNumber.numberWithInt_(0)))
-        self.assertFalse(OC_TestNumber.number_isEqualTo_(42, NSNumber.numberWithLong_(0)))
+        self.assertFalse(
+            OC_TestNumber.number_isEqualTo_(42, NSNumber.numberWithInt_(0))
+        )
+        self.assertFalse(
+            OC_TestNumber.number_isEqualTo_(42, NSNumber.numberWithLong_(0))
+        )
         self.assertFalse(
             OC_TestNumber.number_isEqualTo_(42, NSNumber.numberWithFloat_(0))
         )
-        self.assertFalse(OC_TestNumber.number_isEqualTo_(NSNumber.numberWithInt_(0), 42))
-        self.assertFalse(OC_TestNumber.number_isEqualTo_(NSNumber.numberWithLong_(0), 42))
+        self.assertFalse(
+            OC_TestNumber.number_isEqualTo_(NSNumber.numberWithInt_(0), 42)
+        )
+        self.assertFalse(
+            OC_TestNumber.number_isEqualTo_(NSNumber.numberWithLong_(0), 42)
+        )
         self.assertFalse(
             OC_TestNumber.number_isEqualTo_(NSNumber.numberWithFloat_(0), 42)
         )
@@ -717,5 +746,109 @@ class TestNumberFormatter(TestCase):
         )
 
 
-if __name__ == "__main__":
-    main()
+class FailedComparison:
+    def __eq__(self, other):
+        raise ValueError("comparing is not supported")
+
+    def __ne__(self, other):
+        raise ValueError("comparing is not supported")
+
+
+class NotLess:
+    def __lt__(self, other):
+        return False
+
+
+class NotComparable:
+    def __lt__(self, other):
+        return False
+
+    def __gt__(self, other):
+        return False
+
+
+class FailedComparisonInt(int):
+    def __eq__(self, other):
+        raise ValueError("comparing is not supported")
+
+    def __ne__(self, other):
+        raise ValueError("comparing is not supported")
+
+
+class TestFailedComparisions(TestCase):
+    def test_exception_while_comparing(self):
+        num = 42
+        other = FailedComparison()
+        other_num = FailedComparisonInt(42)
+
+        with self.assertRaisesRegex(ValueError, "comparing is not supported"):
+            OC_Misc.compare_and_(num, other)
+
+        with self.assertRaisesRegex(ValueError, "comparing is not supported"):
+            OC_Misc.compare_and_(other, num)
+
+        with self.assertRaisesRegex(ValueError, "comparing is not supported"):
+            OC_Misc.compare_and_(num, other_num)
+
+        with self.assertRaisesRegex(ValueError, "comparing is not supported"):
+            OC_Misc.compare_and_(other_num, num)
+
+    def test_comparison_failures(self):
+
+        with self.assertRaisesRegex(
+            TypeError, r"'.{1,2}' not supported between instances of 'int' and 'object'"
+        ):
+            OC_Misc.compare_and_(42, object())
+
+        with self.assertRaisesRegex(
+            TypeError, r"'.{1,2}' not supported between instances of 'object' and 'int'"
+        ):
+            OC_Misc.compare_and_(object(), 42)
+
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'.{1,2}' not supported between instances of 'int' and 'NotLess'",
+        ):
+            OC_Misc.compare_and_(42, NotLess())
+
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'.{1,2}' not supported between instances of 'NotLess' and 'int'",
+        ):
+            OC_Misc.compare_and_(NotLess(), 42)
+
+        with self.assertRaisesRegex(TypeError, r".* and .* cannot be compared"):
+            OC_Misc.compare_and_(42, NotComparable())
+
+        with self.assertRaisesRegex(TypeError, r".* and .* cannot be compared"):
+            OC_Misc.compare_and_(NotComparable(), 42)
+
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'.{1,2}' not supported between instances of 'object' and 'object'",
+        ):
+            OC_Misc.compare_and_(object(), object())
+
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'.{1,2}' not supported between instances of 'object' and 'object'",
+        ):
+            OC_Misc.compare_and_(object(), object())
+
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'.{1,2}' not supported between instances of 'object' and 'NotLess'",
+        ):
+            OC_Misc.compare_and_(object(), NotLess())
+
+        with self.assertRaisesRegex(
+            TypeError,
+            r"'.{1,2}' not supported between instances of 'NotLess' and 'object'",
+        ):
+            OC_Misc.compare_and_(NotLess(), object())
+
+        with self.assertRaisesRegex(TypeError, r".* and .* cannot be compared"):
+            OC_Misc.compare_and_(object(), NotComparable())
+
+        with self.assertRaisesRegex(TypeError, r".* and .* cannot be compared"):
+            OC_Misc.compare_and_(NotComparable(), object())
