@@ -1,18 +1,10 @@
 """
 Tests for the proxy of Python sets
 """
-import sys
-from PyObjCTools.TestSupport import *
-from PyObjCTest.fnd import NSSet, NSMutableSet, NSPredicate, NSObject, NSNull
-from PyObjCTest.pythonset import OC_TestSet
 import objc
-
-import os
-
-if sys.version_info[0] == 3:
-    unicode = str
-
-onLeopard = int(os.uname()[2].split(".")[0]) >= 9
+from PyObjCTest.fnd import NSNull, NSObject, NSPredicate
+from PyObjCTest.pythonset import OC_TestSet
+from PyObjCTools.TestSupport import TestCase
 
 OC_PythonSet = objc.lookUpClass("OC_PythonSet")
 OC_BuiltinPythonSet = objc.lookUpClass("OC_BuiltinPythonSet")
@@ -28,8 +20,8 @@ class OC_SetPredicate(NSPredicate):
         self.pred = pred
         return self
 
-    def evaluateWithObject_(self, object):
-        return self.pred(object)
+    def evaluateWithObject_(self, value):
+        return self.pred(value)
 
 
 class OC_TestElem(NSObject):
@@ -100,15 +92,13 @@ class BasicSetTests:
         self.assertFalse(OC_TestSet.set_containsObject_(s, 4))
         self.assertTrue(OC_TestSet.set_containsObject_(s, 2))
 
-    if onLeopard:
+    def testFilteredSetUsingPredicate(self):
+        s = self.setClass(range(10))
+        p = OC_SetPredicate.alloc().initWithFunction_(lambda x: x % 2 == 0)
 
-        def testFilteredSetUsingPredicate(self):
-            s = self.setClass(range(10))
-            p = OC_SetPredicate.alloc().initWithFunction_(lambda x: x % 2 == 0)
-
-            o = OC_TestSet.set_filteredSetUsingPredicate_(s, p)
-            self.assertEqual(o, self.setClass([0, 2, 4, 6, 8]))
-            self.assertEqual(len(s), 10)
+        o = OC_TestSet.set_filteredSetUsingPredicate_(s, p)
+        self.assertEqual(o, self.setClass([0, 2, 4, 6, 8]))
+        self.assertEqual(len(s), 10)
 
     def testMakeObjectsPerform(self):
         o1 = OC_TestElem(1)
@@ -126,22 +116,22 @@ class BasicSetTests:
         s = self.setClass(range(10))
 
         enum = OC_TestSet.objectEnumeratorOfSet_(s)
-        l = []
+        lst = []
         v = enum.nextObject()
         while v is not None:
-            l.append(v)
+            lst.append(v)
             v = enum.nextObject()
-        self.assertEqual(l, list(range(10)))
+        self.assertEqual(lst, list(range(10)))
 
         s = self.setClass([1, 2, None, 3])
         enum = OC_TestSet.objectEnumeratorOfSet_(s)
-        l = []
+        lst = []
         v = enum.nextObject()
         while v is not None:
-            l.append(v)
+            lst.append(v)
             v = enum.nextObject()
 
-        self.assertEqual(dict.fromkeys(l), dict.fromkeys([1, 2, NSNull.null(), 3]))
+        self.assertEqual(dict.fromkeys(lst), dict.fromkeys([1, 2, NSNull.null(), 3]))
 
     def testIsSubSet(self):
         s1 = self.setClass(range(10))
@@ -163,7 +153,7 @@ class BasicSetTests:
     def testDescription(self):
         s = self.setClass([OC_TestElem(1), 2])
         o = OC_TestSet.descriptionOfSet_(s)
-        self.assertIsInstance(o, unicode)
+        self.assertIsInstance(o, str)
 
 
 class TestImmutableSet(TestCase, BasicSetTests):
@@ -181,35 +171,35 @@ class TestImmutableSet(TestCase, BasicSetTests):
     def testNotMutable(self):
         # Ensure that a frozenset cannot be mutated
         o = self.setClass([1, 2, 3])
-        self.assertRaises((TypeError, AttributeError), OC_TestSet.set_addObject_, o, 4)
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_addObject_(o, 4)
 
-        self.assertRaises(TypeError, OC_TestSet.set_removeObject_, o, 2)
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_removeObject_(o, 2)
 
-        self.assertRaises(TypeError, OC_TestSet.set_addObjectsFromArray_, o, [4, 5, 6])
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_addObjectsFromArray_(o, [4, 5, 6])
 
-        if onLeopard:
-            self.assertRaises(
-                TypeError,
-                OC_TestSet.set_filterUsingPredicate_,
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_filterUsingPredicate_(
                 o,
                 NSPredicate.predicateWithValue_(True),
             )
 
-        self.assertRaises(
-            TypeError, OC_TestSet.set_intersectSet_, o, self.setClass([2, 3, 4])
-        )
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_intersectSet_(o, self.setClass([2, 3, 4]))
 
-        self.assertRaises(
-            TypeError, OC_TestSet.set_minusSet_, o, self.setClass([2, 3, 4])
-        )
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_minusSet_(o, self.setClass([2, 3, 4]))
 
-        self.assertRaises(TypeError, OC_TestSet.set_setSet_, o, self.setClass([2, 3, 4]))
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_setSet_(o, self.setClass([2, 3, 4]))
 
-        self.assertRaises(
-            TypeError, OC_TestSet.set_minusSet_, o, self.setClass([2, 3, 4])
-        )
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.set_minusSet_(o, self.setClass([2, 3, 4]))
 
-        self.assertRaises(TypeError, OC_TestSet.removeAllObjecsFromSet_, o)
+        with self.assertRaisesRegex(TypeError, "Cannot mutate a frozenset"):
+            OC_TestSet.removeAllObjecsFromSet_(o)
 
 
 class TestMutableSet(TestCase, BasicSetTests):
@@ -254,14 +244,12 @@ class TestMutableSet(TestCase, BasicSetTests):
         OC_TestSet.set_intersectSet_(s1, s2)
         self.assertEqual(s1, self.setClass([3]))
 
-    if onLeopard:
+    def testFilterSet(self):
+        s = self.setClass(range(10))
+        p = OC_SetPredicate.alloc().initWithFunction_(lambda x: x % 2 == 0)
 
-        def testFilterSet(self):
-            s = self.setClass(range(10))
-            p = OC_SetPredicate.alloc().initWithFunction_(lambda x: x % 2 == 0)
-
-            OC_TestSet.set_filterUsingPredicate_(s, p)
-            self.assertEqual(s, self.setClass([0, 2, 4, 6, 8]))
+        OC_TestSet.set_filterUsingPredicate_(s, p)
+        self.assertEqual(s, self.setClass([0, 2, 4, 6, 8]))
 
     def testAddObject(self):
         s = self.setClass([1, 2, 3])
@@ -295,7 +283,3 @@ class TestMutableSet(TestCase, BasicSetTests):
 
         OC_TestSet.removeAllObjecsFromSet_(s)
         self.assertEqual(s, self.setClass())
-
-
-if __name__ == "__main__":
-    main()
